@@ -14,7 +14,7 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""Session authentication backend"""
+"""Session authentication backend."""
 from __future__ import annotations
 
 from functools import wraps
@@ -26,14 +26,14 @@ CLIENT_AUTH: tuple[str, str] | Any | None = None
 
 
 def init_app(_):
-    """Initializes authentication backend"""
+    """Initialize authentication backend."""
 
 
 T = TypeVar("T", bound=Callable)
 
 
 def requires_authentication(function: T):
-    """Decorator for functions that require authentication"""
+    """Decorate functions that require authentication."""
 
     @wraps(function)
     def decorated(*args, **kwargs):
