@@ -33,15 +33,19 @@ from airflow.api_connexion.schemas.task_instance_schema import (
     TaskInstanceCollection,
     TaskInstanceReferenceCollection,
     clear_task_instance_form,
+    set_single_task_instance_state_form,
+    set_task_instance_note_form_schema,
     set_task_instance_state_form,
     task_instance_batch_form,
     task_instance_collection_schema,
     task_instance_reference_collection_schema,
+    task_instance_reference_schema,
     task_instance_schema,
 )
 from airflow.api_connexion.types import APIResponse
 from airflow.models import SlaMiss
 from airflow.models.dagrun import DagRun as DR
+from airflow.models.operator import needs_expansion
 from airflow.models.taskinstance import TaskInstance as TI, clear_task_instances
 from airflow.security import permissions
 from airflow.utils.airflow_flask_app import get_airflow_app
@@ -66,7 +70,7 @@ def get_task_instance(
     task_id: str,
     session: Session = NEW_SESSION,
 ) -> APIResponse:
-    """Get task instance"""
+    """Get task instance."""
     query = (
         session.query(TI)
         .filter(TI.dag_id == dag_id, TI.run_id == dag_run_id, TI.task_id == task_id)
@@ -115,7 +119,7 @@ def get_mapped_task_instance(
     map_index: int,
     session: Session = NEW_SESSION,
 ) -> APIResponse:
-    """Get task instance"""
+    """Get task instance."""
     query = (
         session.query(TI)
         .filter(
@@ -148,6 +152,8 @@ def get_mapped_task_instance(
         "start_date_lte": format_datetime,
         "end_date_gte": format_datetime,
         "end_date_lte": format_datetime,
+        "updated_at_gte": format_datetime,
+        "updated_at_lte": format_datetime,
     },
 )
 @security.requires_access(
@@ -169,6 +175,8 @@ def get_mapped_task_instances(
     start_date_lte: str | None = None,
     end_date_gte: str | None = None,
     end_date_lte: str | None = None,
+    updated_at_gte: str | None = None,
+    updated_at_lte: str | None = None,
     duration_gte: float | None = None,
     duration_lte: float | None = None,
     state: list[str] | None = None,
@@ -190,7 +198,7 @@ def get_mapped_task_instances(
     )
 
     # 0 can mean a mapped TI that expanded to an empty list, so it is not an automatic 404
-    if base_query.with_entities(func.count('*')).scalar() == 0:
+    if base_query.with_entities(func.count("*")).scalar() == 0:
         dag = get_airflow_app().dag_bag.get_dag(dag_id)
         if not dag:
             error_message = f"DAG {dag_id} not found"
@@ -199,7 +207,7 @@ def get_mapped_task_instances(
         if not task:
             error_message = f"Task id {task_id} not found"
             raise NotFound(error_message)
-        if not task.is_mapped:
+        if not needs_expansion(task):
             error_message = f"Task id {task_id} is not mapped"
             raise NotFound(error_message)
 
@@ -212,12 +220,13 @@ def get_mapped_task_instances(
     query = _apply_range_filter(query, key=TI.start_date, value_range=(start_date_gte, start_date_lte))
     query = _apply_range_filter(query, key=TI.end_date, value_range=(end_date_gte, end_date_lte))
     query = _apply_range_filter(query, key=TI.duration, value_range=(duration_gte, duration_lte))
+    query = _apply_range_filter(query, key=TI.updated_at, value_range=(updated_at_gte, updated_at_lte))
     query = _apply_array_filter(query, key=TI.state, values=states)
     query = _apply_array_filter(query, key=TI.pool, values=pool)
     query = _apply_array_filter(query, key=TI.queue, values=queue)
 
     # Count elements before joining extra columns
-    total_entries = query.with_entities(func.count('*')).scalar()
+    total_entries = query.with_entities(func.count("*")).scalar()
 
     # Add SLA miss
     query = (
@@ -235,11 +244,11 @@ def get_mapped_task_instances(
     )
 
     if order_by:
-        if order_by == 'state':
+        if order_by == "state":
             query = query.order_by(TI.state.asc(), TI.map_index.asc())
-        elif order_by == '-state':
+        elif order_by == "-state":
             query = query.order_by(TI.state.desc(), TI.map_index.asc())
-        elif order_by == '-map_index':
+        elif order_by == "-map_index":
             query = query.order_by(TI.map_index.desc())
         else:
             raise BadRequest(detail=f"Ordering with '{order_by}' is not supported")
@@ -282,6 +291,8 @@ def _apply_range_filter(query: Query, key: ClauseElement, value_range: tuple[T,
         "start_date_lte": format_datetime,
         "end_date_gte": format_datetime,
         "end_date_lte": format_datetime,
+        "updated_at_gte": format_datetime,
+        "updated_at_lte": format_datetime,
     },
 )
 @security.requires_access(
@@ -303,6 +314,8 @@ def get_task_instances(
     start_date_lte: str | None = None,
     end_date_gte: str | None = None,
     end_date_lte: str | None = None,
+    updated_at_gte: str | None = None,
+    updated_at_lte: str | None = None,
     duration_gte: float | None = None,
     duration_lte: float | None = None,
     state: list[str] | None = None,
@@ -331,12 +344,15 @@ def get_task_instances(
     )
     base_query = _apply_range_filter(base_query, key=TI.end_date, value_range=(end_date_gte, end_date_lte))
     base_query = _apply_range_filter(base_query, key=TI.duration, value_range=(duration_gte, duration_lte))
+    base_query = _apply_range_filter(
+        base_query, key=TI.updated_at, value_range=(updated_at_gte, updated_at_lte)
+    )
     base_query = _apply_array_filter(base_query, key=TI.state, values=states)
     base_query = _apply_array_filter(base_query, key=TI.pool, values=pool)
     base_query = _apply_array_filter(base_query, key=TI.queue, values=queue)
 
     # Count elements before joining extra columns
-    total_entries = base_query.with_entities(func.count('*')).scalar()
+    total_entries = base_query.with_entities(func.count("*")).scalar()
     # Add join
     query = (
         base_query.join(
@@ -372,7 +388,7 @@ def get_task_instances_batch(session: Session = NEW_SESSION) -> APIResponse:
         data = task_instance_batch_form.load(body)
     except ValidationError as err:
         raise BadRequest(detail=str(err.messages))
-    states = _convert_state(data['state'])
+    states = _convert_state(data["state"])
     base_query = session.query(TI).join(TI.dag_run)
 
     base_query = _apply_array_filter(base_query, key=TI.dag_id, values=data["dag_ids"])
@@ -397,7 +413,7 @@ def get_task_instances_batch(session: Session = NEW_SESSION) -> APIResponse:
     base_query = _apply_array_filter(base_query, key=TI.queue, values=data["queue"])
 
     # Count elements before joining extra columns
-    total_entries = base_query.with_entities(func.count('*')).scalar()
+    total_entries = base_query.with_entities(func.count("*")).scalar()
     # Add join
     base_query = base_query.join(
         SlaMiss,
@@ -436,28 +452,28 @@ def post_clear_task_instances(*, dag_id: str, session: Session = NEW_SESSION) ->
     if not dag:
         error_message = f"Dag id {dag_id} not found"
         raise NotFound(error_message)
-    reset_dag_runs = data.pop('reset_dag_runs')
-    dry_run = data.pop('dry_run')
+    reset_dag_runs = data.pop("reset_dag_runs")
+    dry_run = data.pop("dry_run")
     # We always pass dry_run here, otherwise this would try to confirm on the terminal!
-    dag_run_id = data.pop('dag_run_id', None)
-    future = data.pop('include_future', False)
-    past = data.pop('include_past', False)
-    downstream = data.pop('include_downstream', False)
-    upstream = data.pop('include_upstream', False)
+    dag_run_id = data.pop("dag_run_id", None)
+    future = data.pop("include_future", False)
+    past = data.pop("include_past", False)
+    downstream = data.pop("include_downstream", False)
+    upstream = data.pop("include_upstream", False)
     if dag_run_id is not None:
         dag_run: DR | None = (
             session.query(DR).filter(DR.dag_id == dag_id, DR.run_id == dag_run_id).one_or_none()
         )
         if dag_run is None:
-            error_message = f'Dag Run id {dag_run_id} not found in dag {dag_id}'
+            error_message = f"Dag Run id {dag_run_id} not found in dag {dag_id}"
             raise NotFound(error_message)
-        data['start_date'] = dag_run.logical_date
-        data['end_date'] = dag_run.logical_date
+        data["start_date"] = dag_run.logical_date
+        data["end_date"] = dag_run.logical_date
     if past:
-        data['start_date'] = None
+        data["start_date"] = None
     if future:
-        data['end_date'] = None
-    task_ids = data.pop('task_ids', None)
+        data["end_date"] = None
+    task_ids = data.pop("task_ids", None)
     if task_ids is not None:
         task_id = [task[0] if isinstance(task, tuple) else task for task in task_ids]
         dag = dag.partial_subset(
@@ -504,15 +520,15 @@ def post_set_task_instances_state(*, dag_id: str, session: Session = NEW_SESSION
     if not dag:
         raise NotFound(error_message)
 
-    task_id = data['task_id']
+    task_id = data["task_id"]
     task = dag.task_dict.get(task_id)
 
     if not task:
         error_message = f"Task ID {task_id} not found"
         raise NotFound(error_message)
 
-    execution_date = data.get('execution_date')
-    run_id = data.get('dag_run_id')
+    execution_date = data.get("execution_date")
+    run_id = data.get("dag_run_id")
     if (
         execution_date
         and (
@@ -526,8 +542,8 @@ def post_set_task_instances_state(*, dag_id: str, session: Session = NEW_SESSION
             detail=f"Task instance not found for task {task_id!r} on execution_date {execution_date}"
         )
 
-    if run_id and not session.query(TI).get(
-        {'task_id': task_id, 'dag_id': dag_id, 'run_id': run_id, 'map_index': -1}
+    if run_id and not session.get(
+        TI, {"task_id": task_id, "dag_id": dag_id, "run_id": run_id, "map_index": -1}
     ):
         error_message = f"Task instance not found for task {task_id!r} on DAG run with ID {run_id!r}"
         raise NotFound(detail=error_message)
@@ -545,3 +561,135 @@ def post_set_task_instances_state(*, dag_id: str, session: Session = NEW_SESSION
         session=session,
     )
     return task_instance_reference_collection_schema.dump(TaskInstanceReferenceCollection(task_instances=tis))
+
+
+def set_mapped_task_instance_note(
+    *, dag_id: str, dag_run_id: str, task_id: str, map_index: int
+) -> APIResponse:
+    """Set the note for a Mapped Task instance."""
+    return set_task_instance_note(dag_id=dag_id, dag_run_id=dag_run_id, task_id=task_id, map_index=map_index)
+
+
+@security.requires_access(
+    [
+        (permissions.ACTION_CAN_EDIT, permissions.RESOURCE_DAG),
+        (permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG_RUN),
+        (permissions.ACTION_CAN_EDIT, permissions.RESOURCE_TASK_INSTANCE),
+    ],
+)
+@provide_session
+def patch_task_instance(
+    *, dag_id: str, dag_run_id: str, task_id: str, map_index: int = -1, session: Session = NEW_SESSION
+) -> APIResponse:
+    """Update the state of a task instance."""
+    body = get_json_request_dict()
+    try:
+        data = set_single_task_instance_state_form.load(body)
+    except ValidationError as err:
+        raise BadRequest(detail=str(err.messages))
+
+    dag = get_airflow_app().dag_bag.get_dag(dag_id)
+    if not dag:
+        raise NotFound("DAG not found", detail=f"DAG {dag_id!r} not found")
+
+    if not dag.has_task(task_id):
+        raise NotFound("Task not found", detail=f"Task {task_id!r} not found in DAG {dag_id!r}")
+
+    ti: TI | None = session.get(
+        TI, {"task_id": task_id, "dag_id": dag_id, "run_id": dag_run_id, "map_index": map_index}
+    )
+
+    if not ti:
+        error_message = f"Task instance not found for task {task_id!r} on DAG run with ID {dag_run_id!r}"
+        raise NotFound(detail=error_message)
+
+    if not data["dry_run"]:
+        ti = dag.set_task_instance_state(
+            task_id=task_id,
+            run_id=dag_run_id,
+            map_indexes=[map_index],
+            state=data["new_state"],
+            commit=True,
+            session=session,
+        )
+
+    return task_instance_reference_schema.dump(ti)
+
+
+@security.requires_access(
+    [
+        (permissions.ACTION_CAN_EDIT, permissions.RESOURCE_DAG),
+        (permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG_RUN),
+        (permissions.ACTION_CAN_EDIT, permissions.RESOURCE_TASK_INSTANCE),
+    ],
+)
+@provide_session
+def patch_mapped_task_instance(
+    *, dag_id: str, dag_run_id: str, task_id: str, map_index: int, session: Session = NEW_SESSION
+) -> APIResponse:
+    """Update the state of a mapped task instance."""
+    return patch_task_instance(
+        dag_id=dag_id, dag_run_id=dag_run_id, task_id=task_id, map_index=map_index, session=session
+    )
+
+
+@security.requires_access(
+    [
+        (permissions.ACTION_CAN_EDIT, permissions.RESOURCE_DAG),
+        (permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG_RUN),
+        (permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG),
+        (permissions.ACTION_CAN_EDIT, permissions.RESOURCE_TASK_INSTANCE),
+    ],
+)
+@provide_session
+def set_task_instance_note(
+    *, dag_id: str, dag_run_id: str, task_id: str, map_index: int = -1, session: Session = NEW_SESSION
+) -> APIResponse:
+    """Set the note for a Task instance. This supports both Mapped and non-Mapped Task instances."""
+    try:
+        post_body = set_task_instance_note_form_schema.load(get_json_request_dict())
+        new_note = post_body["note"]
+    except ValidationError as err:
+        raise BadRequest(detail=str(err))
+
+    query = (
+        session.query(TI)
+        .filter(TI.dag_id == dag_id, TI.run_id == dag_run_id, TI.task_id == task_id)
+        .join(TI.dag_run)
+        .outerjoin(
+            SlaMiss,
+            and_(
+                SlaMiss.dag_id == TI.dag_id,
+                SlaMiss.execution_date == DR.execution_date,
+                SlaMiss.task_id == TI.task_id,
+            ),
+        )
+        .add_entity(SlaMiss)
+        .options(joinedload(TI.rendered_task_instance_fields))
+    )
+    if map_index == -1:
+        query = query.filter(or_(TI.map_index == -1, TI.map_index is None))
+    else:
+        query = query.filter(TI.map_index == map_index)
+
+    try:
+        result = query.one_or_none()
+    except MultipleResultsFound:
+        raise NotFound(
+            "Task instance not found", detail="Task instance is mapped, add the map_index value to the URL"
+        )
+    if result is None:
+        error_message = f"Task Instance not found for dag_id={dag_id}, run_id={dag_run_id}, task_id={task_id}"
+        raise NotFound(error_message)
+
+    ti, sla_miss = result
+    from flask_login import current_user
+
+    current_user_id = getattr(current_user, "id", None)
+    if ti.task_instance_note is None:
+        ti.note = (new_note, current_user_id)
+    else:
+        ti.task_instance_note.content = new_note
+        ti.task_instance_note.user_id = current_user_id
+    session.commit()
+    return task_instance_schema.dump((ti, sla_miss))
