@@ -41,7 +41,7 @@ from airflow.www.fab_security.sqla.models import Role, User
 
 @security.requires_access([(permissions.ACTION_CAN_READ, permissions.RESOURCE_USER)])
 def get_user(*, username: str) -> APIResponse:
-    """Get a user"""
+    """Get a user."""
     ab_security_manager = get_airflow_app().appbuilder.sm
     user = ab_security_manager.find_user(username=username)
     if not user:
@@ -52,7 +52,7 @@ def get_user(*, username: str) -> APIResponse:
 @security.requires_access([(permissions.ACTION_CAN_READ, permissions.RESOURCE_USER)])
 @format_parameters({"limit": check_limit})
 def get_users(*, limit: int, order_by: str = "id", offset: str | None = None) -> APIResponse:
-    """Get users"""
+    """Get users."""
     appbuilder = get_airflow_app().appbuilder
     session = appbuilder.get_session
     total_entries = session.query(func.count(User.id)).scalar()
@@ -61,7 +61,7 @@ def get_users(*, limit: int, order_by: str = "id", offset: str | None = None) ->
     order_param = order_by.strip("-")
     order_param = to_replace.get(order_param, order_param)
     allowed_filter_attrs = [
-        'id',
+        "id",
         "first_name",
         "last_name",
         "user_name",
@@ -83,7 +83,7 @@ def get_users(*, limit: int, order_by: str = "id", offset: str | None = None) ->
 
 @security.requires_access([(permissions.ACTION_CAN_CREATE, permissions.RESOURCE_USER)])
 def post_user() -> APIResponse:
-    """Create a new user"""
+    """Create a new user."""
     try:
         data = user_schema.load(request.json)
     except ValidationError as e:
@@ -119,14 +119,14 @@ def post_user() -> APIResponse:
     user = security_manager.add_user(role=roles_to_add, **data)
     if not user:
         detail = f"Failed to add user `{username}`."
-        return Unknown(detail=detail)
+        raise Unknown(detail=detail)
 
     return user_schema.dump(user)
 
 
 @security.requires_access([(permissions.ACTION_CAN_EDIT, permissions.RESOURCE_USER)])
 def patch_user(*, username: str, update_mask: UpdateMask = None) -> APIResponse:
-    """Update a user"""
+    """Update a user."""
     try:
         data = user_schema.load(request.json)
     except ValidationError as e:
@@ -139,13 +139,13 @@ def patch_user(*, username: str, update_mask: UpdateMask = None) -> APIResponse:
         detail = f"The User with username `{username}` was not found"
         raise NotFound(title="User not found", detail=detail)
     # Check unique username
-    new_username = data.get('username')
+    new_username = data.get("username")
     if new_username and new_username != username:
         if security_manager.find_user(username=new_username):
             raise AlreadyExists(detail=f"The username `{new_username}` already exists")
 
     # Check unique email
-    email = data.get('email')
+    email = data.get("email")
     if email and email != user.email:
         if security_manager.find_user(email=email):
             raise AlreadyExists(detail=f"The email `{email}` already exists")
@@ -195,7 +195,7 @@ def patch_user(*, username: str, update_mask: UpdateMask = None) -> APIResponse:
 
 @security.requires_access([(permissions.ACTION_CAN_DELETE, permissions.RESOURCE_USER)])
 def delete_user(*, username: str) -> APIResponse:
-    """Delete a user"""
+    """Delete a user."""
     security_manager = get_airflow_app().appbuilder.sm
 
     user = security_manager.find_user(username=username)
