@@ -29,10 +29,12 @@ from termcolor import colored
 
 from airflow.configuration import AIRFLOW_HOME, conf, make_group_other_inaccessible
 from airflow.executors import executor_constants
-from airflow.jobs.scheduler_job import SchedulerJob
-from airflow.jobs.triggerer_job import TriggererJob
+from airflow.executors.executor_loader import ExecutorLoader
+from airflow.jobs.base_job_runner import BaseJobRunner
+from airflow.jobs.job import most_recent_job
+from airflow.jobs.scheduler_job_runner import SchedulerJobRunner
+from airflow.jobs.triggerer_job_runner import TriggererJobRunner
 from airflow.utils import db
-from airflow.www.app import cached_app
 
 
 class StandaloneCommand:
@@ -155,11 +157,10 @@ class StandaloneCommand:
         We override some settings as part of being standalone.
         """
         env = dict(os.environ)
+
         # Make sure we're using a local executor flavour
-        if conf.get("core", "executor") not in [
-            executor_constants.LOCAL_EXECUTOR,
-            executor_constants.SEQUENTIAL_EXECUTOR,
-        ]:
+        executor_class, _ = ExecutorLoader.import_default_executor_cls()
+        if not executor_class.is_local:
             if "sqlite" in conf.get("database", "sql_alchemy_conn"):
                 self.print_output("standalone", "Forcing executor to SequentialExecutor")
                 env["AIRFLOW__CORE__EXECUTOR"] = executor_constants.SEQUENTIAL_EXECUTOR
@@ -180,8 +181,10 @@ class StandaloneCommand:
         # server. Thus, we make a random password and store it in AIRFLOW_HOME,
         # with the reasoning that if you can read that directory, you can see
         # the database credentials anyway.
-        appbuilder = cached_app().appbuilder
-        user_exists = appbuilder.sm.find_user("admin")
+        from airflow.utils.cli_app_builder import get_application_builder
+
+        with get_application_builder() as appbuilder:
+            user_exists = appbuilder.sm.find_user("admin")
         password_path = os.path.join(AIRFLOW_HOME, "standalone_admin_password.txt")
         we_know_password = os.path.isfile(password_path)
         # If the user does not exist, make a random password and make it
@@ -214,8 +217,8 @@ class StandaloneCommand:
         """
         return (
             self.port_open(self.web_server_port)
-            and self.job_running(SchedulerJob)
-            and self.job_running(TriggererJob)
+            and self.job_running(SchedulerJobRunner)
+            and self.job_running(TriggererJobRunner)
         )
 
     def port_open(self, port):
@@ -234,13 +237,13 @@ class StandaloneCommand:
             return False
         return True
 
-    def job_running(self, job):
+    def job_running(self, job_runner_class: type[BaseJobRunner]):
         """
         Checks if the given job name is running and heartbeating correctly.
 
         Used to tell if scheduler is alive.
         """
-        recent = job.most_recent_job()
+        recent = most_recent_job(job_runner_class.job_type)
         if not recent:
             return False
         return recent.is_alive()
