@@ -41,6 +41,7 @@ from airflow.models.abstractoperator import (
     DEFAULT_RETRIES,
     DEFAULT_RETRY_DELAY,
     DEFAULT_TRIGGER_RULE,
+    DEFAULT_WAIT_FOR_PAST_DEPENDS_BEFORE_SKIPPING,
     DEFAULT_WEIGHT_RULE,
     AbstractOperator,
     NotMapped,
@@ -65,6 +66,7 @@ from airflow.utils.helpers import is_container, prevent_duplicates
 from airflow.utils.operator_resources import Resources
 from airflow.utils.trigger_rule import TriggerRule
 from airflow.utils.types import NOTSET
+from airflow.utils.xcom import XCOM_RETURN_KEY
 
 if TYPE_CHECKING:
     import jinja2  # Slow import.
@@ -110,7 +112,7 @@ def validate_mapping_kwargs(op: type[BaseOperator], func: ValidationSource, valu
 
 
 def ensure_xcomarg_return_value(arg: Any) -> None:
-    from airflow.models.xcom_arg import XCOM_RETURN_KEY, XComArg
+    from airflow.models.xcom_arg import XComArg
 
     if isinstance(arg, XComArg):
         for operator, key in arg.iter_references():
@@ -390,6 +392,13 @@ class MappedOperator(AbstractOperator):
         value = self.partial_kwargs.get("ignore_first_depends_on_past", DEFAULT_IGNORE_FIRST_DEPENDS_ON_PAST)
         return bool(value)
 
+    @property
+    def wait_for_past_depends_before_skipping(self) -> bool:
+        value = self.partial_kwargs.get(
+            "wait_for_past_depends_before_skipping", DEFAULT_WAIT_FOR_PAST_DEPENDS_BEFORE_SKIPPING
+        )
+        return bool(value)
+
     @property
     def wait_for_downstream(self) -> bool:
         return bool(self.partial_kwargs.get("wait_for_downstream"))
@@ -442,12 +451,16 @@ class MappedOperator(AbstractOperator):
     def max_active_tis_per_dag(self) -> int | None:
         return self.partial_kwargs.get("max_active_tis_per_dag")
 
+    @property
+    def max_active_tis_per_dagrun(self) -> int | None:
+        return self.partial_kwargs.get("max_active_tis_per_dagrun")
+
     @property
     def resources(self) -> Resources | None:
         return self.partial_kwargs.get("resources")
 
     @property
-    def on_execute_callback(self) -> TaskStateChangeCallback | None:
+    def on_execute_callback(self) -> None | TaskStateChangeCallback | list[TaskStateChangeCallback]:
         return self.partial_kwargs.get("on_execute_callback")
 
     @on_execute_callback.setter
@@ -455,7 +468,7 @@ class MappedOperator(AbstractOperator):
         self.partial_kwargs["on_execute_callback"] = value
 
     @property
-    def on_failure_callback(self) -> TaskStateChangeCallback | None:
+    def on_failure_callback(self) -> None | TaskStateChangeCallback | list[TaskStateChangeCallback]:
         return self.partial_kwargs.get("on_failure_callback")
 
     @on_failure_callback.setter
@@ -463,7 +476,7 @@ class MappedOperator(AbstractOperator):
         self.partial_kwargs["on_failure_callback"] = value
 
     @property
-    def on_retry_callback(self) -> TaskStateChangeCallback | None:
+    def on_retry_callback(self) -> None | TaskStateChangeCallback | list[TaskStateChangeCallback]:
         return self.partial_kwargs.get("on_retry_callback")
 
     @on_retry_callback.setter
@@ -471,7 +484,7 @@ class MappedOperator(AbstractOperator):
         self.partial_kwargs["on_retry_callback"] = value
 
     @property
-    def on_success_callback(self) -> TaskStateChangeCallback | None:
+    def on_success_callback(self) -> None | TaskStateChangeCallback | list[TaskStateChangeCallback]:
         return self.partial_kwargs.get("on_success_callback")
 
     @on_success_callback.setter
