@@ -15,9 +15,10 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
 
-from typing import TYPE_CHECKING, Optional, Sequence
-from urllib.parse import urlparse
+from typing import TYPE_CHECKING, Sequence
+from urllib.parse import urlsplit
 
 from airflow.compat.functools import cached_property
 from airflow.exceptions import AirflowException
@@ -42,14 +43,14 @@ class OSSKeySensor(BaseSensorOperator):
     :param oss_conn_id: The Airflow connection used for OSS credentials.
     """
 
-    template_fields: Sequence[str] = ('bucket_key', 'bucket_name')
+    template_fields: Sequence[str] = ("bucket_key", "bucket_name")
 
     def __init__(
         self,
         bucket_key: str,
         region: str,
-        bucket_name: Optional[str] = None,
-        oss_conn_id: Optional[str] = 'oss_default',
+        bucket_name: str | None = None,
+        oss_conn_id: str | None = "oss_default",
         **kwargs,
     ):
         super().__init__(**kwargs)
@@ -58,9 +59,9 @@ class OSSKeySensor(BaseSensorOperator):
         self.bucket_key = bucket_key
         self.region = region
         self.oss_conn_id = oss_conn_id
-        self.hook: Optional[OSSHook] = None
+        self.hook: OSSHook | None = None
 
-    def poke(self, context: 'Context'):
+    def poke(self, context: Context):
         """
         Check if the object exists in the bucket to pull key.
         @param self - the object itself
@@ -68,21 +69,21 @@ class OSSKeySensor(BaseSensorOperator):
         @returns True if the object exists, False otherwise
         """
         if self.bucket_name is None:
-            parsed_url = urlparse(self.bucket_key)
-            if parsed_url.netloc == '':
-                raise AirflowException('If key is a relative path from root, please provide a bucket_name')
+            parsed_url = urlsplit(self.bucket_key)
+            if parsed_url.netloc == "":
+                raise AirflowException("If key is a relative path from root, please provide a bucket_name")
             self.bucket_name = parsed_url.netloc
-            self.bucket_key = parsed_url.path.lstrip('/')
+            self.bucket_key = parsed_url.path.lstrip("/")
         else:
-            parsed_url = urlparse(self.bucket_key)
-            if parsed_url.scheme != '' or parsed_url.netloc != '':
+            parsed_url = urlsplit(self.bucket_key)
+            if parsed_url.scheme != "" or parsed_url.netloc != "":
                 raise AirflowException(
-                    'If bucket_name is provided, bucket_key'
-                    ' should be relative path from root'
-                    ' level, rather than a full oss:// url'
+                    "If bucket_name is provided, bucket_key"
+                    " should be relative path from root"
+                    " level, rather than a full oss:// url"
                 )
 
-        self.log.info('Poking for key : oss://%s/%s', self.bucket_name, self.bucket_key)
+        self.log.info("Poking for key : oss://%s/%s", self.bucket_name, self.bucket_key)
         return self.get_hook.object_exists(key=self.bucket_key, bucket_name=self.bucket_name)
 
     @cached_property
