@@ -15,13 +15,16 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
 
 import os.path
-from typing import TYPE_CHECKING, Optional, Sequence
+import urllib.parse
+from typing import TYPE_CHECKING, Sequence
 
 from airflow.models import BaseOperator
 from airflow.providers.amazon.aws.hooks.glue import GlueJobHook
 from airflow.providers.amazon.aws.hooks.s3 import S3Hook
+from airflow.providers.amazon.aws.links.glue import GlueJobRunDetailsLink
 
 if TYPE_CHECKING:
     from airflow.utils.context import Context
@@ -51,40 +54,45 @@ class GlueJobOperator(BaseOperator):
     :param run_job_kwargs: Extra arguments for Glue Job Run
     :param wait_for_completion: Whether or not wait for job run completion. (default: True)
     :param verbose: If True, Glue Job Run logs show in the Airflow Task Logs.  (default: False)
+    :param update_config: If True, Operator will update job configuration.  (default: False)
     """
 
     template_fields: Sequence[str] = (
-        'job_name',
-        'script_location',
-        'script_args',
-        's3_bucket',
-        'iam_role_name',
+        "job_name",
+        "script_location",
+        "script_args",
+        "create_job_kwargs",
+        "s3_bucket",
+        "iam_role_name",
     )
     template_ext: Sequence[str] = ()
     template_fields_renderers = {
         "script_args": "json",
         "create_job_kwargs": "json",
     }
-    ui_color = '#ededed'
+    ui_color = "#ededed"
+
+    operator_extra_links = (GlueJobRunDetailsLink(),)
 
     def __init__(
         self,
         *,
-        job_name: str = 'aws_glue_default_job',
-        job_desc: str = 'AWS Glue Job with Airflow',
-        script_location: Optional[str] = None,
-        concurrent_run_limit: Optional[int] = None,
-        script_args: Optional[dict] = None,
+        job_name: str = "aws_glue_default_job",
+        job_desc: str = "AWS Glue Job with Airflow",
+        script_location: str | None = None,
+        concurrent_run_limit: int | None = None,
+        script_args: dict | None = None,
         retry_limit: int = 0,
-        num_of_dpus: Optional[int] = None,
-        aws_conn_id: str = 'aws_default',
-        region_name: Optional[str] = None,
-        s3_bucket: Optional[str] = None,
-        iam_role_name: Optional[str] = None,
-        create_job_kwargs: Optional[dict] = None,
-        run_job_kwargs: Optional[dict] = None,
+        num_of_dpus: int | float | None = None,
+        aws_conn_id: str = "aws_default",
+        region_name: str | None = None,
+        s3_bucket: str | None = None,
+        iam_role_name: str | None = None,
+        create_job_kwargs: dict | None = None,
+        run_job_kwargs: dict | None = None,
         wait_for_completion: bool = True,
         verbose: bool = False,
+        update_config: bool = False,
         **kwargs,
     ):
         super().__init__(**kwargs)
@@ -100,13 +108,14 @@ class GlueJobOperator(BaseOperator):
         self.s3_bucket = s3_bucket
         self.iam_role_name = iam_role_name
         self.s3_protocol = "s3://"
-        self.s3_artifacts_prefix = 'artifacts/glue-scripts/'
+        self.s3_artifacts_prefix = "artifacts/glue-scripts/"
         self.create_job_kwargs = create_job_kwargs
         self.run_job_kwargs = run_job_kwargs or {}
         self.wait_for_completion = wait_for_completion
         self.verbose = verbose
+        self.update_config = update_config
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         """
         Executes AWS Glue Job from Airflow
 
@@ -135,6 +144,7 @@ class GlueJobOperator(BaseOperator):
             s3_bucket=self.s3_bucket,
             iam_role_name=self.iam_role_name,
             create_job_kwargs=self.create_job_kwargs,
+            update_config=self.update_config,
         )
         self.log.info(
             "Initializing AWS Glue Job: %s. Wait for completion: %s",
@@ -142,14 +152,29 @@ class GlueJobOperator(BaseOperator):
             self.wait_for_completion,
         )
         glue_job_run = glue_job.initialize_job(self.script_args, self.run_job_kwargs)
+        glue_job_run_url = GlueJobRunDetailsLink.format_str.format(
+            aws_domain=GlueJobRunDetailsLink.get_aws_domain(glue_job.conn_partition),
+            region_name=glue_job.conn_region_name,
+            job_name=urllib.parse.quote(self.job_name, safe=""),
+            job_run_id=glue_job_run["JobRunId"],
+        )
+        GlueJobRunDetailsLink.persist(
+            context=context,
+            operator=self,
+            region_name=glue_job.conn_region_name,
+            aws_partition=glue_job.conn_partition,
+            job_name=urllib.parse.quote(self.job_name, safe=""),
+            job_run_id=glue_job_run["JobRunId"],
+        )
+        self.log.info("You can monitor this Glue Job run at: %s", glue_job_run_url)
         if self.wait_for_completion:
-            glue_job_run = glue_job.job_completion(self.job_name, glue_job_run['JobRunId'], self.verbose)
+            glue_job_run = glue_job.job_completion(self.job_name, glue_job_run["JobRunId"], self.verbose)
             self.log.info(
                 "AWS Glue Job: %s status: %s. Run Id: %s",
                 self.job_name,
-                glue_job_run['JobRunState'],
-                glue_job_run['JobRunId'],
+                glue_job_run["JobRunState"],
+                glue_job_run["JobRunId"],
             )
         else:
-            self.log.info("AWS Glue Job: %s. Run Id: %s", self.job_name, glue_job_run['JobRunId'])
-        return glue_job_run['JobRunId']
+            self.log.info("AWS Glue Job: %s. Run Id: %s", self.job_name, glue_job_run["JobRunId"])
+        return glue_job_run["JobRunId"]
