@@ -19,6 +19,9 @@ from __future__ import annotations
 import time
 from typing import TYPE_CHECKING, Sequence
 
+from deprecated import deprecated
+
+from airflow.compat.functools import cached_property
 from airflow.exceptions import AirflowException
 from airflow.providers.amazon.aws.hooks.sagemaker import LogState, SageMakerHook
 from airflow.sensors.base import BaseSensorOperator
@@ -37,30 +40,34 @@ class SageMakerBaseSensor(BaseSensorOperator):
 
     ui_color = "#ededed"
 
-    def __init__(self, *, aws_conn_id: str = "aws_default", **kwargs):
+    def __init__(self, *, aws_conn_id: str = "aws_default", resource_type: str = "job", **kwargs):
         super().__init__(**kwargs)
         self.aws_conn_id = aws_conn_id
-        self.hook: SageMakerHook | None = None
+        self.resource_type = resource_type  # only used for logs, to say what kind of resource we are sensing
 
+    @deprecated(reason="use `hook` property instead.")
     def get_hook(self) -> SageMakerHook:
         """Get SageMakerHook."""
-        if self.hook:
-            return self.hook
-        self.hook = SageMakerHook(aws_conn_id=self.aws_conn_id)
         return self.hook
 
+    @cached_property
+    def hook(self) -> SageMakerHook:
+        return SageMakerHook(aws_conn_id=self.aws_conn_id)
+
     def poke(self, context: Context):
         response = self.get_sagemaker_response()
         if response["ResponseMetadata"]["HTTPStatusCode"] != 200:
             self.log.info("Bad HTTP response: %s", response)
             return False
         state = self.state_from_response(response)
-        self.log.info("Job currently %s", state)
+        self.log.info("%s currently %s", self.resource_type, state)
         if state in self.non_terminal_states():
             return False
         if state in self.failed_states():
             failed_reason = self.get_failed_reason_from_response(response)
-            raise AirflowException(f"Sagemaker job failed for the following reason: {failed_reason}")
+            raise AirflowException(
+                f"Sagemaker {self.resource_type} failed for the following reason: {failed_reason}"
+            )
         return True
 
     def non_terminal_states(self) -> set[str]:
@@ -111,7 +118,7 @@ class SageMakerEndpointSensor(SageMakerBaseSensor):
 
     def get_sagemaker_response(self):
         self.log.info("Poking Sagemaker Endpoint %s", self.endpoint_name)
-        return self.get_hook().describe_endpoint(self.endpoint_name)
+        return self.hook.describe_endpoint(self.endpoint_name)
 
     def get_failed_reason_from_response(self, response):
         return response["FailureReason"]
@@ -147,7 +154,7 @@ class SageMakerTransformSensor(SageMakerBaseSensor):
 
     def get_sagemaker_response(self):
         self.log.info("Poking Sagemaker Transform Job %s", self.job_name)
-        return self.get_hook().describe_transform_job(self.job_name)
+        return self.hook.describe_transform_job(self.job_name)
 
     def get_failed_reason_from_response(self, response):
         return response["FailureReason"]
@@ -183,7 +190,7 @@ class SageMakerTuningSensor(SageMakerBaseSensor):
 
     def get_sagemaker_response(self):
         self.log.info("Poking Sagemaker Tuning Job %s", self.job_name)
-        return self.get_hook().describe_tuning_job(self.job_name)
+        return self.hook.describe_tuning_job(self.job_name)
 
     def get_failed_reason_from_response(self, response):
         return response["FailureReason"]
@@ -240,12 +247,12 @@ class SageMakerTrainingSensor(SageMakerBaseSensor):
     def get_sagemaker_response(self):
         if self.print_log:
             if not self.log_resource_inited:
-                self.init_log_resource(self.get_hook())
+                self.init_log_resource(self.hook)
             (
                 self.state,
                 self.last_description,
                 self.last_describe_job_call,
-            ) = self.get_hook().describe_training_job_with_log(
+            ) = self.hook.describe_training_job_with_log(
                 self.job_name,
                 self.positions,
                 self.stream_names,
@@ -255,7 +262,7 @@ class SageMakerTrainingSensor(SageMakerBaseSensor):
                 self.last_describe_job_call,
             )
         else:
-            self.last_description = self.get_hook().describe_training_job(self.job_name)
+            self.last_description = self.hook.describe_training_job(self.job_name)
         status = self.state_from_response(self.last_description)
         if (status not in self.non_terminal_states()) and (status not in self.failed_states()):
             billable_time = (
@@ -269,3 +276,70 @@ class SageMakerTrainingSensor(SageMakerBaseSensor):
 
     def state_from_response(self, response):
         return response["TrainingJobStatus"]
+
+
+class SageMakerPipelineSensor(SageMakerBaseSensor):
+    """
+    Polls the pipeline until it reaches a terminal state.  Raises an
+    AirflowException with the failure reason if a failed state is reached.
+
+    .. seealso::
+        For more information on how to use this sensor, take a look at the guide:
+        :ref:`howto/sensor:SageMakerPipelineSensor`
+
+    :param pipeline_exec_arn: ARN of the pipeline to watch.
+    :param verbose: Whether to print steps details while waiting for completion.
+            Defaults to true, consider turning off for pipelines that have thousands of steps.
+    """
+
+    template_fields: Sequence[str] = ("pipeline_exec_arn",)
+
+    def __init__(self, *, pipeline_exec_arn: str, verbose: bool = True, **kwargs):
+        super().__init__(resource_type="pipeline", **kwargs)
+        self.pipeline_exec_arn = pipeline_exec_arn
+        self.verbose = verbose
+
+    def non_terminal_states(self) -> set[str]:
+        return SageMakerHook.pipeline_non_terminal_states
+
+    def failed_states(self) -> set[str]:
+        return SageMakerHook.failed_states
+
+    def get_sagemaker_response(self) -> dict:
+        self.log.info("Poking Sagemaker Pipeline Execution %s", self.pipeline_exec_arn)
+        return self.hook.describe_pipeline_exec(self.pipeline_exec_arn, self.verbose)
+
+    def state_from_response(self, response: dict) -> str:
+        return response["PipelineExecutionStatus"]
+
+
+class SageMakerAutoMLSensor(SageMakerBaseSensor):
+    """
+    Polls the auto ML job until it reaches a terminal state.
+    Raises an AirflowException with the failure reason if a failed state is reached.
+
+    .. seealso::
+        For more information on how to use this sensor, take a look at the guide:
+        :ref:`howto/sensor:SageMakerAutoMLSensor`
+
+    :param job_name: unique name of the AutoML job to watch.
+    """
+
+    template_fields: Sequence[str] = ("job_name",)
+
+    def __init__(self, *, job_name: str, **kwargs):
+        super().__init__(resource_type="autoML job", **kwargs)
+        self.job_name = job_name
+
+    def non_terminal_states(self) -> set[str]:
+        return SageMakerHook.non_terminal_states
+
+    def failed_states(self) -> set[str]:
+        return SageMakerHook.failed_states
+
+    def get_sagemaker_response(self) -> dict:
+        self.log.info("Poking Sagemaker AutoML Execution %s", self.job_name)
+        return self.hook._describe_auto_ml_job(self.job_name)
+
+    def state_from_response(self, response: dict) -> str:
+        return response["AutoMLJobStatus"]
