@@ -18,6 +18,7 @@ from __future__ import annotations
 
 import shlex
 import time
+import warnings
 from io import StringIO
 from typing import Any
 
@@ -28,12 +29,15 @@ from airflow.compat.functools import cached_property
 from airflow.providers.google.cloud.hooks.compute import ComputeEngineHook
 from airflow.providers.google.cloud.hooks.os_login import OSLoginHook
 from airflow.providers.ssh.hooks.ssh import SSHHook
+from airflow.utils.types import NOTSET, ArgNotSet
 
 # Paramiko should be imported after airflow.providers.ssh. Then the import will fail with
 # cannot import "airflow.providers.ssh" and will be correctly discovered as optional feature
 # TODO:(potiuk) We should add test harness detecting such cases shortly
 import paramiko  # isort:skip
 
+CMD_TIMEOUT = 10
+
 
 class _GCloudAuthorizedSSHClient(paramiko.SSHClient):
     """SSH Client that maintains the context for gcloud authorization during the connection"""
@@ -109,6 +113,8 @@ class ComputeEngineSSHHook(SSHHook):
         use_oslogin: bool = True,
         expire_time: int = 300,
         delegate_to: str | None = None,
+        cmd_timeout: int | ArgNotSet = NOTSET,
+        **kwargs,
     ) -> None:
         # Ignore original constructor
         # super().__init__()
@@ -122,7 +128,12 @@ class ComputeEngineSSHHook(SSHHook):
         self.use_oslogin = use_oslogin
         self.expire_time = expire_time
         self.gcp_conn_id = gcp_conn_id
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         self.delegate_to = delegate_to
+        self.cmd_timeout = cmd_timeout
         self._conn: Any | None = None
 
     @cached_property
@@ -174,6 +185,17 @@ class ComputeEngineSSHHook(SSHHook):
                 self.expire_time,
             )
 
+            if conn.extra is not None:
+                extra_options = conn.extra_dejson
+                if "cmd_timeout" in extra_options and self.cmd_timeout is NOTSET:
+                    if extra_options["cmd_timeout"]:
+                        self.cmd_timeout = int(extra_options["cmd_timeout"])
+                    else:
+                        self.cmd_timeout = None
+
+            if self.cmd_timeout is NOTSET:
+                self.cmd_timeout = CMD_TIMEOUT
+
     def get_conn(self) -> paramiko.SSHClient:
         """Return SSH connection."""
         self._load_connection_config()
