@@ -16,7 +16,10 @@
 # specific language governing permissions and limitations
 # under the License.
 """Hook for Secrets Manager service"""
-from typing import Optional, Sequence, Union
+from __future__ import annotations
+
+import warnings
+from typing import Sequence
 
 from airflow.providers.google.cloud._internal_client.secret_manager_client import _SecretManagerClient
 from airflow.providers.google.common.hooks.base_google import GoogleBaseHook
@@ -34,7 +37,7 @@ class SecretsManagerHook(GoogleBaseHook):
     :param gcp_conn_id: The connection ID to use when fetching connection info.
     :param delegate_to: The account to impersonate using domain-wide delegation of authority,
         if any. For this to work, the service account making the request must have
-        domain-wide delegation enabled.
+        domain-wide delegation enabled. (Deprecated)
     :param impersonation_chain: Optional service account to impersonate using short-term
         credentials, or chained list of accounts required to get the access_token
         of the last account in the list, which will be impersonated in the request.
@@ -48,9 +51,13 @@ class SecretsManagerHook(GoogleBaseHook):
     def __init__(
         self,
         gcp_conn_id: str = "google_cloud_default",
-        delegate_to: Optional[str] = None,
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        delegate_to: str | None = None,
+        impersonation_chain: str | Sequence[str] | None = None,
     ) -> None:
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         super().__init__(
             gcp_conn_id=gcp_conn_id,
             delegate_to=delegate_to,
@@ -63,14 +70,13 @@ class SecretsManagerHook(GoogleBaseHook):
         Retrieves the connection to Secret Manager.
 
         :return: Secret Manager client.
-        :rtype: airflow.providers.google.cloud._internal_client.secret_manager_client._SecretManagerClient
         """
         return self.client
 
     @GoogleBaseHook.fallback_to_default_project_id
     def get_secret(
-        self, secret_id: str, secret_version: str = 'latest', project_id: Optional[str] = None
-    ) -> Optional[str]:
+        self, secret_id: str, secret_version: str = "latest", project_id: str | None = None
+    ) -> str | None:
         """
         Get secret value from the Secret Manager.
 
