@@ -15,7 +15,10 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-from typing import TYPE_CHECKING, Dict, Optional, Sequence, Tuple, Union
+from __future__ import annotations
+
+import warnings
+from typing import TYPE_CHECKING, Sequence
 
 from google.api_core.exceptions import AlreadyExists
 from google.api_core.gapic_v1.method import DEFAULT, _MethodDefault
@@ -25,9 +28,9 @@ from google.cloud.orchestration.airflow.service_v1.types import Environment
 from google.protobuf.field_mask_pb2 import FieldMask
 
 from airflow import AirflowException
-from airflow.models import BaseOperator
 from airflow.providers.google.cloud.hooks.cloud_composer import CloudComposerHook
 from airflow.providers.google.cloud.links.base import BaseGoogleLink
+from airflow.providers.google.cloud.operators.cloud_base import GoogleCloudBaseOperator
 from airflow.providers.google.cloud.triggers.cloud_composer import CloudComposerExecutionTrigger
 from airflow.providers.google.common.consts import GOOGLE_DEFAULT_DEFERRABLE_METHOD_NAME
 
@@ -38,7 +41,7 @@ CLOUD_COMPOSER_BASE_LINK = "https://console.cloud.google.com/composer/environmen
 CLOUD_COMPOSER_DETAILS_LINK = (
     CLOUD_COMPOSER_BASE_LINK + "/detail/{region}/{environment_id}/monitoring?project={project_id}"
 )
-CLOUD_COMPOSER_ENVIRONMENTS_LINK = CLOUD_COMPOSER_BASE_LINK + '?project={project_id}'
+CLOUD_COMPOSER_ENVIRONMENTS_LINK = CLOUD_COMPOSER_BASE_LINK + "?project={project_id}"
 
 
 class CloudComposerEnvironmentLink(BaseGoogleLink):
@@ -50,12 +53,12 @@ class CloudComposerEnvironmentLink(BaseGoogleLink):
 
     @staticmethod
     def persist(
-        operator_instance: Union[
-            "CloudComposerCreateEnvironmentOperator",
-            "CloudComposerUpdateEnvironmentOperator",
-            "CloudComposerGetEnvironmentOperator",
-        ],
-        context: "Context",
+        operator_instance: (
+            CloudComposerCreateEnvironmentOperator
+            | CloudComposerUpdateEnvironmentOperator
+            | CloudComposerGetEnvironmentOperator
+        ),
+        context: Context,
     ) -> None:
         operator_instance.xcom_push(
             context,
@@ -76,7 +79,7 @@ class CloudComposerEnvironmentsLink(BaseGoogleLink):
     format_str = CLOUD_COMPOSER_ENVIRONMENTS_LINK
 
     @staticmethod
-    def persist(operator_instance: "CloudComposerListEnvironmentsOperator", context: "Context") -> None:
+    def persist(operator_instance: CloudComposerListEnvironmentsOperator, context: Context) -> None:
         operator_instance.xcom_push(
             context,
             key=CloudComposerEnvironmentsLink.key,
@@ -86,7 +89,7 @@ class CloudComposerEnvironmentsLink(BaseGoogleLink):
         )
 
 
-class CloudComposerCreateEnvironmentOperator(BaseOperator):
+class CloudComposerCreateEnvironmentOperator(GoogleCloudBaseOperator):
     """
     Create a new environment.
 
@@ -115,11 +118,11 @@ class CloudComposerCreateEnvironmentOperator(BaseOperator):
     """
 
     template_fields = (
-        'project_id',
-        'region',
-        'environment_id',
-        'environment',
-        'impersonation_chain',
+        "project_id",
+        "region",
+        "environment_id",
+        "environment",
+        "impersonation_chain",
     )
 
     operator_extra_links = (CloudComposerEnvironmentLink(),)
@@ -130,13 +133,13 @@ class CloudComposerCreateEnvironmentOperator(BaseOperator):
         project_id: str,
         region: str,
         environment_id: str,
-        environment: Union[Environment, Dict],
+        environment: Environment | dict,
         gcp_conn_id: str = "google_cloud_default",
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
-        delegate_to: Optional[str] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        impersonation_chain: str | Sequence[str] | None = None,
+        delegate_to: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
         deferrable: bool = False,
         pooling_period_seconds: int = 30,
         **kwargs,
@@ -151,11 +154,15 @@ class CloudComposerCreateEnvironmentOperator(BaseOperator):
         self.metadata = metadata
         self.gcp_conn_id = gcp_conn_id
         self.impersonation_chain = impersonation_chain
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         self.delegate_to = delegate_to
         self.deferrable = deferrable
         self.pooling_period_seconds = pooling_period_seconds
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = CloudComposerHook(
             gcp_conn_id=self.gcp_conn_id,
             impersonation_chain=self.impersonation_chain,
@@ -178,6 +185,8 @@ class CloudComposerCreateEnvironmentOperator(BaseOperator):
                 timeout=self.timeout,
                 metadata=self.metadata,
             )
+            context["ti"].xcom_push(key="operation_id", value=result.operation.name)
+
             if not self.deferrable:
                 environment = hook.wait_for_operation(timeout=self.timeout, operation=result)
                 return Environment.to_dict(environment)
@@ -205,7 +214,7 @@ class CloudComposerCreateEnvironmentOperator(BaseOperator):
             )
             return Environment.to_dict(environment)
 
-    def execute_complete(self, context: "Context", event: dict):
+    def execute_complete(self, context: Context, event: dict):
         if event["operation_done"]:
             hook = CloudComposerHook(
                 gcp_conn_id=self.gcp_conn_id,
@@ -226,7 +235,7 @@ class CloudComposerCreateEnvironmentOperator(BaseOperator):
             raise AirflowException(f"Unexpected error in the operation: {event['operation_name']}")
 
 
-class CloudComposerDeleteEnvironmentOperator(BaseOperator):
+class CloudComposerDeleteEnvironmentOperator(GoogleCloudBaseOperator):
     """
     Delete an environment.
 
@@ -254,10 +263,10 @@ class CloudComposerDeleteEnvironmentOperator(BaseOperator):
     """
 
     template_fields = (
-        'project_id',
-        'region',
-        'environment_id',
-        'impersonation_chain',
+        "project_id",
+        "region",
+        "environment_id",
+        "impersonation_chain",
     )
 
     def __init__(
@@ -266,12 +275,12 @@ class CloudComposerDeleteEnvironmentOperator(BaseOperator):
         project_id: str,
         region: str,
         environment_id: str,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
         gcp_conn_id: str = "google_cloud_default",
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
-        delegate_to: Optional[str] = None,
+        impersonation_chain: str | Sequence[str] | None = None,
+        delegate_to: str | None = None,
         deferrable: bool = False,
         pooling_period_seconds: int = 30,
         **kwargs,
@@ -285,11 +294,15 @@ class CloudComposerDeleteEnvironmentOperator(BaseOperator):
         self.metadata = metadata
         self.gcp_conn_id = gcp_conn_id
         self.impersonation_chain = impersonation_chain
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         self.delegate_to = delegate_to
         self.deferrable = deferrable
         self.pooling_period_seconds = pooling_period_seconds
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = CloudComposerHook(
             gcp_conn_id=self.gcp_conn_id,
             impersonation_chain=self.impersonation_chain,
@@ -319,11 +332,11 @@ class CloudComposerDeleteEnvironmentOperator(BaseOperator):
                 method_name=GOOGLE_DEFAULT_DEFERRABLE_METHOD_NAME,
             )
 
-    def execute_complete(self, context: "Context", event: dict):
+    def execute_complete(self, context: Context, event: dict):
         pass
 
 
-class CloudComposerGetEnvironmentOperator(BaseOperator):
+class CloudComposerGetEnvironmentOperator(GoogleCloudBaseOperator):
     """
     Get an existing environment.
 
@@ -348,10 +361,10 @@ class CloudComposerGetEnvironmentOperator(BaseOperator):
     """
 
     template_fields = (
-        'project_id',
-        'region',
-        'environment_id',
-        'impersonation_chain',
+        "project_id",
+        "region",
+        "environment_id",
+        "impersonation_chain",
     )
 
     operator_extra_links = (CloudComposerEnvironmentLink(),)
@@ -362,12 +375,12 @@ class CloudComposerGetEnvironmentOperator(BaseOperator):
         project_id: str,
         region: str,
         environment_id: str,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
         gcp_conn_id: str = "google_cloud_default",
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
-        delegate_to: Optional[str] = None,
+        impersonation_chain: str | Sequence[str] | None = None,
+        delegate_to: str | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -379,9 +392,13 @@ class CloudComposerGetEnvironmentOperator(BaseOperator):
         self.metadata = metadata
         self.gcp_conn_id = gcp_conn_id
         self.impersonation_chain = impersonation_chain
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         self.delegate_to = delegate_to
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = CloudComposerHook(
             gcp_conn_id=self.gcp_conn_id,
             impersonation_chain=self.impersonation_chain,
@@ -401,7 +418,7 @@ class CloudComposerGetEnvironmentOperator(BaseOperator):
         return Environment.to_dict(result)
 
 
-class CloudComposerListEnvironmentsOperator(BaseOperator):
+class CloudComposerListEnvironmentsOperator(GoogleCloudBaseOperator):
     """
     List environments.
 
@@ -428,9 +445,9 @@ class CloudComposerListEnvironmentsOperator(BaseOperator):
     """
 
     template_fields = (
-        'project_id',
-        'region',
-        'impersonation_chain',
+        "project_id",
+        "region",
+        "impersonation_chain",
     )
 
     operator_extra_links = (CloudComposerEnvironmentsLink(),)
@@ -440,14 +457,14 @@ class CloudComposerListEnvironmentsOperator(BaseOperator):
         *,
         project_id: str,
         region: str,
-        page_size: Optional[int] = None,
-        page_token: Optional[str] = None,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        page_size: int | None = None,
+        page_token: str | None = None,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
         gcp_conn_id: str = "google_cloud_default",
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
-        delegate_to: Optional[str] = None,
+        impersonation_chain: str | Sequence[str] | None = None,
+        delegate_to: str | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -460,9 +477,13 @@ class CloudComposerListEnvironmentsOperator(BaseOperator):
         self.metadata = metadata
         self.gcp_conn_id = gcp_conn_id
         self.impersonation_chain = impersonation_chain
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         self.delegate_to = delegate_to
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = CloudComposerHook(
             gcp_conn_id=self.gcp_conn_id,
             impersonation_chain=self.impersonation_chain,
@@ -481,7 +502,7 @@ class CloudComposerListEnvironmentsOperator(BaseOperator):
         return [Environment.to_dict(env) for env in result]
 
 
-class CloudComposerUpdateEnvironmentOperator(BaseOperator):
+class CloudComposerUpdateEnvironmentOperator(GoogleCloudBaseOperator):
     r"""
     Update an environment.
 
@@ -514,10 +535,10 @@ class CloudComposerUpdateEnvironmentOperator(BaseOperator):
     """
 
     template_fields = (
-        'project_id',
-        'region',
-        'environment_id',
-        'impersonation_chain',
+        "project_id",
+        "region",
+        "environment_id",
+        "impersonation_chain",
     )
 
     operator_extra_links = (CloudComposerEnvironmentLink(),)
@@ -528,14 +549,14 @@ class CloudComposerUpdateEnvironmentOperator(BaseOperator):
         project_id: str,
         region: str,
         environment_id: str,
-        environment: Union[Dict, Environment],
-        update_mask: Union[Dict, FieldMask],
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        environment: dict | Environment,
+        update_mask: dict | FieldMask,
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
         gcp_conn_id: str = "google_cloud_default",
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
-        delegate_to: Optional[str] = None,
+        impersonation_chain: str | Sequence[str] | None = None,
+        delegate_to: str | None = None,
         deferrable: bool = False,
         pooling_period_seconds: int = 30,
         **kwargs,
@@ -551,11 +572,15 @@ class CloudComposerUpdateEnvironmentOperator(BaseOperator):
         self.metadata = metadata
         self.gcp_conn_id = gcp_conn_id
         self.impersonation_chain = impersonation_chain
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         self.delegate_to = delegate_to
         self.deferrable = deferrable
         self.pooling_period_seconds = pooling_period_seconds
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = CloudComposerHook(
             gcp_conn_id=self.gcp_conn_id,
             impersonation_chain=self.impersonation_chain,
@@ -591,7 +616,7 @@ class CloudComposerUpdateEnvironmentOperator(BaseOperator):
                 method_name=GOOGLE_DEFAULT_DEFERRABLE_METHOD_NAME,
             )
 
-    def execute_complete(self, context: "Context", event: dict):
+    def execute_complete(self, context: Context, event: dict):
         if event["operation_done"]:
             hook = CloudComposerHook(
                 gcp_conn_id=self.gcp_conn_id,
@@ -612,7 +637,7 @@ class CloudComposerUpdateEnvironmentOperator(BaseOperator):
             raise AirflowException(f"Unexpected error in the operation: {event['operation_name']}")
 
 
-class CloudComposerListImageVersionsOperator(BaseOperator):
+class CloudComposerListImageVersionsOperator(GoogleCloudBaseOperator):
     """
     List ImageVersions for provided location.
 
@@ -635,9 +660,9 @@ class CloudComposerListImageVersionsOperator(BaseOperator):
     """
 
     template_fields = (
-        'project_id',
-        'region',
-        'impersonation_chain',
+        "project_id",
+        "region",
+        "impersonation_chain",
     )
 
     def __init__(
@@ -645,15 +670,15 @@ class CloudComposerListImageVersionsOperator(BaseOperator):
         *,
         project_id: str,
         region: str,
-        page_size: Optional[int] = None,
-        page_token: Optional[str] = None,
+        page_size: int | None = None,
+        page_token: str | None = None,
         include_past_releases: bool = False,
-        retry: Union[Retry, _MethodDefault] = DEFAULT,
-        timeout: Optional[float] = None,
-        metadata: Sequence[Tuple[str, str]] = (),
+        retry: Retry | _MethodDefault = DEFAULT,
+        timeout: float | None = None,
+        metadata: Sequence[tuple[str, str]] = (),
         gcp_conn_id: str = "google_cloud_default",
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
-        delegate_to: Optional[str] = None,
+        impersonation_chain: str | Sequence[str] | None = None,
+        delegate_to: str | None = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -667,9 +692,13 @@ class CloudComposerListImageVersionsOperator(BaseOperator):
         self.metadata = metadata
         self.gcp_conn_id = gcp_conn_id
         self.impersonation_chain = impersonation_chain
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         self.delegate_to = delegate_to
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = CloudComposerHook(
             gcp_conn_id=self.gcp_conn_id,
             impersonation_chain=self.impersonation_chain,
