@@ -27,13 +27,13 @@ from google.cloud.compute_v1.types import Instance, InstanceGroupManager, Instan
 from json_merge_patch import merge
 
 from airflow.exceptions import AirflowException
-from airflow.models import BaseOperator
 from airflow.providers.google.cloud.hooks.compute import ComputeEngineHook
 from airflow.providers.google.cloud.links.compute import (
     ComputeInstanceDetailsLink,
     ComputeInstanceGroupManagerDetailsLink,
     ComputeInstanceTemplateDetailsLink,
 )
+from airflow.providers.google.cloud.operators.cloud_base import GoogleCloudBaseOperator
 from airflow.providers.google.cloud.utils.field_sanitizer import GcpBodyFieldSanitizer
 from airflow.providers.google.cloud.utils.field_validator import GcpBodyFieldValidator
 
@@ -41,7 +41,7 @@ if TYPE_CHECKING:
     from airflow.utils.context import Context
 
 
-class ComputeEngineBaseOperator(BaseOperator):
+class ComputeEngineBaseOperator(GoogleCloudBaseOperator):
     """Abstract base operator for Google Compute Engine operators to inherit from."""
 
     def __init__(
@@ -944,6 +944,7 @@ class ComputeEngineInsertInstanceTemplateOperator(ComputeEngineBaseOperator):
         )
         self._validate_all_body_fields()
         self.check_body_fields()
+        self._field_sanitizer.sanitize(self.body)
         try:
             # Idempotence check (sort of) - we want to check if the new Template
             # is already created and if is, then we assume it was created by previous run
@@ -1093,7 +1094,7 @@ class ComputeEngineDeleteInstanceTemplateOperator(ComputeEngineBaseOperator):
                 project_id=self.project_id,
                 request_id=self.request_id,
             )
-            self.log.info("Successfully deleted Instance template")
+            self.log.info("Successfully deleted Instance template %s", self.resource_id)
         except exceptions.NotFound as e:
             # Expecting 404 Error in case if Instance template doesn't exist.
             if e.code == 404:
@@ -1246,7 +1247,7 @@ class ComputeEngineCopyInstanceTemplateOperator(ComputeEngineBaseOperator):
         new_body = merge(new_body, self.body_patch)
         self.log.info("Calling insert instance template with updated body: %s", new_body)
         hook.insert_instance_template(body=new_body, request_id=self.request_id, project_id=self.project_id)
-        instance_template = hook.get_instance_template(
+        new_instance_tmp = hook.get_instance_template(
             resource_id=self.body_patch["name"], project_id=self.project_id
         )
         ComputeInstanceTemplateDetailsLink.persist(
@@ -1255,7 +1256,7 @@ class ComputeEngineCopyInstanceTemplateOperator(ComputeEngineBaseOperator):
             resource_id=self.body_patch["name"],
             project_id=self.project_id or hook.project_id,
         )
-        return InstanceTemplate.to_dict(instance_template)
+        return InstanceTemplate.to_dict(new_instance_tmp)
 
 
 class ComputeEngineInstanceGroupUpdateManagerTemplateOperator(ComputeEngineBaseOperator):
