@@ -16,18 +16,20 @@
 # specific language governing permissions and limitations
 # under the License.
 """This module contains Google Spanner operators."""
-from typing import TYPE_CHECKING, List, Optional, Sequence, Union
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Sequence
 
 from airflow.exceptions import AirflowException
-from airflow.models import BaseOperator
 from airflow.providers.google.cloud.hooks.spanner import SpannerHook
 from airflow.providers.google.cloud.links.spanner import SpannerDatabaseLink, SpannerInstanceLink
+from airflow.providers.google.cloud.operators.cloud_base import GoogleCloudBaseOperator
 
 if TYPE_CHECKING:
     from airflow.utils.context import Context
 
 
-class SpannerDeployInstanceOperator(BaseOperator):
+class SpannerDeployInstanceOperator(GoogleCloudBaseOperator):
     """
     Creates a new Cloud Spanner instance, or if an instance with the same instance_id
     exists in the specified project, updates the Cloud Spanner instance.
@@ -60,12 +62,12 @@ class SpannerDeployInstanceOperator(BaseOperator):
 
     # [START gcp_spanner_deploy_template_fields]
     template_fields: Sequence[str] = (
-        'project_id',
-        'instance_id',
-        'configuration_name',
-        'display_name',
-        'gcp_conn_id',
-        'impersonation_chain',
+        "project_id",
+        "instance_id",
+        "configuration_name",
+        "display_name",
+        "gcp_conn_id",
+        "impersonation_chain",
     )
     # [END gcp_spanner_deploy_template_fields]
     operator_extra_links = (SpannerInstanceLink(),)
@@ -77,9 +79,9 @@ class SpannerDeployInstanceOperator(BaseOperator):
         configuration_name: str,
         node_count: int,
         display_name: str,
-        project_id: Optional[str] = None,
-        gcp_conn_id: str = 'google_cloud_default',
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        project_id: str | None = None,
+        gcp_conn_id: str = "google_cloud_default",
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         self.instance_id = instance_id
@@ -93,12 +95,12 @@ class SpannerDeployInstanceOperator(BaseOperator):
         super().__init__(**kwargs)
 
     def _validate_inputs(self) -> None:
-        if self.project_id == '':
+        if self.project_id == "":
             raise AirflowException("The required parameter 'project_id' is empty")
         if not self.instance_id:
             raise AirflowException("The required parameter 'instance_id' is empty or None")
 
-    def execute(self, context: 'Context') -> None:
+    def execute(self, context: Context) -> None:
         hook = SpannerHook(
             gcp_conn_id=self.gcp_conn_id,
             impersonation_chain=self.impersonation_chain,
@@ -124,7 +126,7 @@ class SpannerDeployInstanceOperator(BaseOperator):
         )
 
 
-class SpannerDeleteInstanceOperator(BaseOperator):
+class SpannerDeleteInstanceOperator(GoogleCloudBaseOperator):
     """
     Deletes a Cloud Spanner instance. If an instance does not exist,
     no action is taken and the operator succeeds.
@@ -149,10 +151,10 @@ class SpannerDeleteInstanceOperator(BaseOperator):
 
     # [START gcp_spanner_delete_template_fields]
     template_fields: Sequence[str] = (
-        'project_id',
-        'instance_id',
-        'gcp_conn_id',
-        'impersonation_chain',
+        "project_id",
+        "instance_id",
+        "gcp_conn_id",
+        "impersonation_chain",
     )
     # [END gcp_spanner_delete_template_fields]
 
@@ -160,9 +162,9 @@ class SpannerDeleteInstanceOperator(BaseOperator):
         self,
         *,
         instance_id: str,
-        project_id: Optional[str] = None,
-        gcp_conn_id: str = 'google_cloud_default',
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        project_id: str | None = None,
+        gcp_conn_id: str = "google_cloud_default",
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         self.instance_id = instance_id
@@ -173,12 +175,12 @@ class SpannerDeleteInstanceOperator(BaseOperator):
         super().__init__(**kwargs)
 
     def _validate_inputs(self) -> None:
-        if self.project_id == '':
+        if self.project_id == "":
             raise AirflowException("The required parameter 'project_id' is empty")
         if not self.instance_id:
             raise AirflowException("The required parameter 'instance_id' is empty or None")
 
-    def execute(self, context: 'Context') -> Optional[bool]:
+    def execute(self, context: Context) -> bool | None:
         hook = SpannerHook(
             gcp_conn_id=self.gcp_conn_id,
             impersonation_chain=self.impersonation_chain,
@@ -194,7 +196,7 @@ class SpannerDeleteInstanceOperator(BaseOperator):
             return True
 
 
-class SpannerQueryDatabaseInstanceOperator(BaseOperator):
+class SpannerQueryDatabaseInstanceOperator(GoogleCloudBaseOperator):
     """
     Executes an arbitrary DML query (INSERT, UPDATE, DELETE).
 
@@ -221,15 +223,15 @@ class SpannerQueryDatabaseInstanceOperator(BaseOperator):
 
     # [START gcp_spanner_query_template_fields]
     template_fields: Sequence[str] = (
-        'project_id',
-        'instance_id',
-        'database_id',
-        'query',
-        'gcp_conn_id',
-        'impersonation_chain',
+        "project_id",
+        "instance_id",
+        "database_id",
+        "query",
+        "gcp_conn_id",
+        "impersonation_chain",
     )
-    template_ext: Sequence[str] = ('.sql',)
-    template_fields_renderers = {'query': 'sql'}
+    template_ext: Sequence[str] = (".sql",)
+    template_fields_renderers = {"query": "sql"}
     # [END gcp_spanner_query_template_fields]
     operator_extra_links = (SpannerDatabaseLink(),)
 
@@ -238,10 +240,10 @@ class SpannerQueryDatabaseInstanceOperator(BaseOperator):
         *,
         instance_id: str,
         database_id: str,
-        query: Union[str, List[str]],
-        project_id: Optional[str] = None,
-        gcp_conn_id: str = 'google_cloud_default',
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        query: str | list[str],
+        project_id: str | None = None,
+        gcp_conn_id: str = "google_cloud_default",
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         self.instance_id = instance_id
@@ -254,7 +256,7 @@ class SpannerQueryDatabaseInstanceOperator(BaseOperator):
         super().__init__(**kwargs)
 
     def _validate_inputs(self) -> None:
-        if self.project_id == '':
+        if self.project_id == "":
             raise AirflowException("The required parameter 'project_id' is empty")
         if not self.instance_id:
             raise AirflowException("The required parameter 'instance_id' is empty or None")
@@ -263,13 +265,13 @@ class SpannerQueryDatabaseInstanceOperator(BaseOperator):
         if not self.query:
             raise AirflowException("The required parameter 'query' is empty")
 
-    def execute(self, context: 'Context'):
+    def execute(self, context: Context):
         hook = SpannerHook(
             gcp_conn_id=self.gcp_conn_id,
             impersonation_chain=self.impersonation_chain,
         )
         if isinstance(self.query, str):
-            queries = [x.strip() for x in self.query.split(';')]
+            queries = [x.strip() for x in self.query.split(";")]
             self.sanitize_queries(queries)
         else:
             queries = self.query
@@ -295,18 +297,17 @@ class SpannerQueryDatabaseInstanceOperator(BaseOperator):
         )
 
     @staticmethod
-    def sanitize_queries(queries: List[str]) -> None:
+    def sanitize_queries(queries: list[str]) -> None:
         """
         Drops empty query in queries.
 
         :param queries: queries
-        :rtype: None
         """
-        if queries and queries[-1] == '':
+        if queries and queries[-1] == "":
             del queries[-1]
 
 
-class SpannerDeployDatabaseInstanceOperator(BaseOperator):
+class SpannerDeployDatabaseInstanceOperator(GoogleCloudBaseOperator):
     """
     Creates a new Cloud Spanner database, or if database exists,
     the operator does nothing.
@@ -333,15 +334,15 @@ class SpannerDeployDatabaseInstanceOperator(BaseOperator):
 
     # [START gcp_spanner_database_deploy_template_fields]
     template_fields: Sequence[str] = (
-        'project_id',
-        'instance_id',
-        'database_id',
-        'ddl_statements',
-        'gcp_conn_id',
-        'impersonation_chain',
+        "project_id",
+        "instance_id",
+        "database_id",
+        "ddl_statements",
+        "gcp_conn_id",
+        "impersonation_chain",
     )
-    template_ext: Sequence[str] = ('.sql',)
-    template_fields_renderers = {'ddl_statements': 'sql'}
+    template_ext: Sequence[str] = (".sql",)
+    template_fields_renderers = {"ddl_statements": "sql"}
     # [END gcp_spanner_database_deploy_template_fields]
     operator_extra_links = (SpannerDatabaseLink(),)
 
@@ -350,10 +351,10 @@ class SpannerDeployDatabaseInstanceOperator(BaseOperator):
         *,
         instance_id: str,
         database_id: str,
-        ddl_statements: List[str],
-        project_id: Optional[str] = None,
-        gcp_conn_id: str = 'google_cloud_default',
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        ddl_statements: list[str],
+        project_id: str | None = None,
+        gcp_conn_id: str = "google_cloud_default",
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         self.instance_id = instance_id
@@ -366,14 +367,14 @@ class SpannerDeployDatabaseInstanceOperator(BaseOperator):
         super().__init__(**kwargs)
 
     def _validate_inputs(self) -> None:
-        if self.project_id == '':
+        if self.project_id == "":
             raise AirflowException("The required parameter 'project_id' is empty")
         if not self.instance_id:
             raise AirflowException("The required parameter 'instance_id' is empty or None")
         if not self.database_id:
             raise AirflowException("The required parameter 'database_id' is empty or None")
 
-    def execute(self, context: 'Context') -> Optional[bool]:
+    def execute(self, context: Context) -> bool | None:
         hook = SpannerHook(
             gcp_conn_id=self.gcp_conn_id,
             impersonation_chain=self.impersonation_chain,
@@ -411,7 +412,7 @@ class SpannerDeployDatabaseInstanceOperator(BaseOperator):
         return True
 
 
-class SpannerUpdateDatabaseInstanceOperator(BaseOperator):
+class SpannerUpdateDatabaseInstanceOperator(GoogleCloudBaseOperator):
     """
     Updates a Cloud Spanner database with the specified DDL statement.
 
@@ -439,15 +440,15 @@ class SpannerUpdateDatabaseInstanceOperator(BaseOperator):
 
     # [START gcp_spanner_database_update_template_fields]
     template_fields: Sequence[str] = (
-        'project_id',
-        'instance_id',
-        'database_id',
-        'ddl_statements',
-        'gcp_conn_id',
-        'impersonation_chain',
+        "project_id",
+        "instance_id",
+        "database_id",
+        "ddl_statements",
+        "gcp_conn_id",
+        "impersonation_chain",
     )
-    template_ext: Sequence[str] = ('.sql',)
-    template_fields_renderers = {'ddl_statements': 'sql'}
+    template_ext: Sequence[str] = (".sql",)
+    template_fields_renderers = {"ddl_statements": "sql"}
     # [END gcp_spanner_database_update_template_fields]
     operator_extra_links = (SpannerDatabaseLink(),)
 
@@ -456,11 +457,11 @@ class SpannerUpdateDatabaseInstanceOperator(BaseOperator):
         *,
         instance_id: str,
         database_id: str,
-        ddl_statements: List[str],
-        project_id: Optional[str] = None,
-        operation_id: Optional[str] = None,
-        gcp_conn_id: str = 'google_cloud_default',
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        ddl_statements: list[str],
+        project_id: str | None = None,
+        operation_id: str | None = None,
+        gcp_conn_id: str = "google_cloud_default",
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         self.instance_id = instance_id
@@ -474,7 +475,7 @@ class SpannerUpdateDatabaseInstanceOperator(BaseOperator):
         super().__init__(**kwargs)
 
     def _validate_inputs(self) -> None:
-        if self.project_id == '':
+        if self.project_id == "":
             raise AirflowException("The required parameter 'project_id' is empty")
         if not self.instance_id:
             raise AirflowException("The required parameter 'instance_id' is empty or None")
@@ -483,7 +484,7 @@ class SpannerUpdateDatabaseInstanceOperator(BaseOperator):
         if not self.ddl_statements:
             raise AirflowException("The required parameter 'ddl_statements' is empty or None")
 
-    def execute(self, context: 'Context') -> None:
+    def execute(self, context: Context) -> None:
         hook = SpannerHook(
             gcp_conn_id=self.gcp_conn_id,
             impersonation_chain=self.impersonation_chain,
@@ -513,7 +514,7 @@ class SpannerUpdateDatabaseInstanceOperator(BaseOperator):
             )
 
 
-class SpannerDeleteDatabaseInstanceOperator(BaseOperator):
+class SpannerDeleteDatabaseInstanceOperator(GoogleCloudBaseOperator):
     """
     Deletes a Cloud Spanner database.
 
@@ -538,11 +539,11 @@ class SpannerDeleteDatabaseInstanceOperator(BaseOperator):
 
     # [START gcp_spanner_database_delete_template_fields]
     template_fields: Sequence[str] = (
-        'project_id',
-        'instance_id',
-        'database_id',
-        'gcp_conn_id',
-        'impersonation_chain',
+        "project_id",
+        "instance_id",
+        "database_id",
+        "gcp_conn_id",
+        "impersonation_chain",
     )
     # [END gcp_spanner_database_delete_template_fields]
 
@@ -551,9 +552,9 @@ class SpannerDeleteDatabaseInstanceOperator(BaseOperator):
         *,
         instance_id: str,
         database_id: str,
-        project_id: Optional[str] = None,
-        gcp_conn_id: str = 'google_cloud_default',
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        project_id: str | None = None,
+        gcp_conn_id: str = "google_cloud_default",
+        impersonation_chain: str | Sequence[str] | None = None,
         **kwargs,
     ) -> None:
         self.instance_id = instance_id
@@ -565,14 +566,14 @@ class SpannerDeleteDatabaseInstanceOperator(BaseOperator):
         super().__init__(**kwargs)
 
     def _validate_inputs(self) -> None:
-        if self.project_id == '':
+        if self.project_id == "":
             raise AirflowException("The required parameter 'project_id' is empty")
         if not self.instance_id:
             raise AirflowException("The required parameter 'instance_id' is empty or None")
         if not self.database_id:
             raise AirflowException("The required parameter 'database_id' is empty or None")
 
-    def execute(self, context: 'Context') -> bool:
+    def execute(self, context: Context) -> bool:
         hook = SpannerHook(
             gcp_conn_id=self.gcp_conn_id,
             impersonation_chain=self.impersonation_chain,
