@@ -19,11 +19,22 @@
 from __future__ import annotations
 
 import json
+import warnings
 from typing import TYPE_CHECKING, Any, Sequence
 
-from google.api_core.exceptions import Conflict
+from google.api_core.exceptions import BadRequest, Conflict
 from google.api_core.retry import Retry
-from google.cloud.bigquery import DEFAULT_RETRY, CopyJob, ExtractJob, LoadJob, QueryJob
+from google.cloud.bigquery import (
+    DEFAULT_RETRY,
+    CopyJob,
+    ExternalConfig,
+    ExtractJob,
+    LoadJob,
+    QueryJob,
+    SchemaField,
+    UnknownJob,
+)
+from google.cloud.bigquery.table import EncryptionConfiguration, Table, TableReference
 
 from airflow import AirflowException
 from airflow.models import BaseOperator
@@ -35,6 +46,15 @@ from airflow.providers.google.cloud.triggers.bigquery import BigQueryInsertJobTr
 if TYPE_CHECKING:
     from airflow.utils.context import Context
 
+ALLOWED_FORMATS = [
+    "CSV",
+    "NEWLINE_DELIMITED_JSON",
+    "AVRO",
+    "GOOGLE_SHEETS",
+    "DATASTORE_BACKUP",
+    "PARQUET",
+]
+
 
 class GCSToBigQueryOperator(BaseOperator):
     """
@@ -129,6 +149,8 @@ class GCSToBigQueryOperator(BaseOperator):
         options and schema for CSV and JSON sources. (Default: ``True``).
         Parameter must be set to True if 'schema_fields' and 'schema_object' are undefined.
         It is suggested to set to True if table are create outside of Airflow.
+        If autodetect is None and no schema is provided (neither via schema_fields
+        nor a schema_object), assume the table already exists.
     :param encryption_configuration: [Optional] Custom encryption configuration (e.g., Cloud KMS keys).
         **Example**: ::
 
@@ -206,6 +228,7 @@ class GCSToBigQueryOperator(BaseOperator):
         job_id: str | None = None,
         force_rerun: bool = True,
         reattach_states: set[str] | None = None,
+        project_id: str | None = None,
         **kwargs,
     ) -> None:
 
@@ -228,8 +251,15 @@ class GCSToBigQueryOperator(BaseOperator):
 
         # BQ config
         self.destination_project_dataset_table = destination_project_dataset_table
+        self.project_id = project_id
         self.schema_fields = schema_fields
-        self.source_format = source_format
+        if source_format.upper() not in ALLOWED_FORMATS:
+            raise ValueError(
+                f"{source_format} is not a valid source format. "
+                f"Please use one of the following types: {ALLOWED_FORMATS}."
+            )
+        else:
+            self.source_format = source_format.upper()
         self.compression = compression
         self.create_disposition = create_disposition
         self.skip_leading_rows = skip_leading_rows
@@ -245,6 +275,10 @@ class GCSToBigQueryOperator(BaseOperator):
 
         self.max_id_key = max_id_key
         self.gcp_conn_id = gcp_conn_id
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         self.delegate_to = delegate_to
 
         self.schema_update_options = schema_update_options
@@ -275,7 +309,7 @@ class GCSToBigQueryOperator(BaseOperator):
         # Submit a new job without waiting for it to complete.
         return hook.insert_job(
             configuration=self.configuration,
-            project_id=hook.project_id,
+            project_id=self.project_id,
             location=self.location,
             job_id=job_id,
             timeout=self.result_timeout,
@@ -284,7 +318,7 @@ class GCSToBigQueryOperator(BaseOperator):
         )
 
     @staticmethod
-    def _handle_job_error(job: BigQueryJob) -> None:
+    def _handle_job_error(job: BigQueryJob | UnknownJob) -> None:
         if job.error_result:
             raise AirflowException(f"BigQuery job {job.job_id} failed: {job.error_result}")
 
@@ -296,6 +330,8 @@ class GCSToBigQueryOperator(BaseOperator):
             impersonation_chain=self.impersonation_chain,
         )
         self.hook = hook
+        self.source_format = self.source_format.upper()
+
         job_id = self.hook.generate_job_id(
             job_id=self.job_id,
             dag_id=self.dag_id,
@@ -308,94 +344,50 @@ class GCSToBigQueryOperator(BaseOperator):
         self.source_objects = (
             self.source_objects if isinstance(self.source_objects, list) else [self.source_objects]
         )
-        source_uris = [f"gs://{self.bucket}/{source_object}" for source_object in self.source_objects]
+        self.source_uris = [f"gs://{self.bucket}/{source_object}" for source_object in self.source_objects]
+
         if not self.schema_fields:
-            gcs_hook = GCSHook(
-                gcp_conn_id=self.gcp_conn_id,
-                delegate_to=self.delegate_to,
-                impersonation_chain=self.impersonation_chain,
-            )
+            # Check for self.autodetect explicitly False. self.autodetect equal to None
+            # entails we do not want to detect schema from files. Instead, it means we
+            # rely on an already existing table's schema
+            if not self.schema_object and self.autodetect is False:
+                raise AirflowException(
+                    "Table schema was not found. Neither schema object nor schema fields were specified"
+                )
             if self.schema_object and self.source_format != "DATASTORE_BACKUP":
-                schema_fields = json.loads(gcs_hook.download(self.bucket, self.schema_object).decode("utf-8"))
-                self.log.info("Autodetected fields from schema object: %s", schema_fields)
+                gcs_hook = GCSHook(
+                    gcp_conn_id=self.gcp_conn_id,
+                    delegate_to=self.delegate_to,
+                    impersonation_chain=self.impersonation_chain,
+                )
+                self.schema_fields = json.loads(
+                    gcs_hook.download(self.schema_object_bucket, self.schema_object).decode("utf-8")
+                )
+                self.log.info("Loaded fields from schema object: %s", self.schema_fields)
+            else:
+                self.schema_fields = None
 
         if self.external_table:
             self.log.info("Creating a new BigQuery table for storing data...")
-            project_id, dataset_id, table_id = self.hook.split_tablename(
-                table_input=self.destination_project_dataset_table,
-                default_project_id=self.hook.project_id or "",
-            )
-            table_resource = {
-                "tableReference": {
-                    "projectId": project_id,
-                    "datasetId": dataset_id,
-                    "tableId": table_id,
-                },
-                "labels": self.labels,
-                "description": self.description,
-                "externalDataConfiguration": {
-                    "source_uris": source_uris,
-                    "source_format": self.source_format,
-                    "maxBadRecords": self.max_bad_records,
-                    "autodetect": self.autodetect,
-                    "compression": self.compression,
-                    "csvOptions": {
-                        "fieldDelimeter": self.field_delimiter,
-                        "skipLeadingRows": self.skip_leading_rows,
-                        "quote": self.quote_character,
-                        "allowQuotedNewlines": self.allow_quoted_newlines,
-                        "allowJaggedRows": self.allow_jagged_rows,
-                    },
-                },
-                "location": self.location,
-                "encryptionConfiguration": self.encryption_configuration,
-            }
-            table_resource_checked_schema = self._check_schema_fields(table_resource)
-            table = self.hook.create_empty_table(
-                table_resource=table_resource_checked_schema,
-            )
-            max_id = self._find_max_value_in_column()
+            table_obj_api_repr = self._create_empty_table()
+
             BigQueryTableLink.persist(
                 context=context,
                 task_instance=self,
-                dataset_id=table.to_api_repr()["tableReference"]["datasetId"],
-                project_id=table.to_api_repr()["tableReference"]["projectId"],
-                table_id=table.to_api_repr()["tableReference"]["tableId"],
+                dataset_id=table_obj_api_repr["tableReference"]["datasetId"],
+                project_id=table_obj_api_repr["tableReference"]["projectId"],
+                table_id=table_obj_api_repr["tableReference"]["tableId"],
             )
-            return max_id
+            if self.max_id_key:
+                max_id = self._find_max_value_in_column()
+                return max_id
         else:
             self.log.info("Using existing BigQuery table for storing data...")
-            destination_project, destination_dataset, destination_table = self.hook.split_tablename(
-                table_input=self.destination_project_dataset_table,
-                default_project_id=self.hook.project_id or "",
-                var_name="destination_project_dataset_table",
-            )
-            self.configuration = {
-                "load": {
-                    "autodetect": self.autodetect,
-                    "createDisposition": self.create_disposition,
-                    "destinationTable": {
-                        "projectId": destination_project,
-                        "datasetId": destination_dataset,
-                        "tableId": destination_table,
-                    },
-                    "destinationTableProperties": {
-                        "description": self.description,
-                        "labels": self.labels,
-                    },
-                    "sourceFormat": self.source_format,
-                    "skipLeadingRows": self.skip_leading_rows,
-                    "sourceUris": source_uris,
-                    "writeDisposition": self.write_disposition,
-                    "ignoreUnknownValues": self.ignore_unknown_values,
-                    "allowQuotedNewlines": self.allow_quoted_newlines,
-                    "encoding": self.encoding,
-                },
-            }
-            self.configuration = self._check_schema_fields(self.configuration)
+            self.configuration = self._use_existing_table()
+
             try:
                 self.log.info("Executing: %s", self.configuration)
-                job = self._submit_job(self.hook, job_id)
+                job: BigQueryJob | UnknownJob = self._submit_job(self.hook, job_id)
             except Conflict:
                 # If the job already exists retrieve it
                 job = self.hook.get_job(
@@ -454,9 +446,9 @@ class GCSToBigQueryOperator(BaseOperator):
                 )
             else:
                 job.result(timeout=self.result_timeout, retry=self.result_retry)
-                max_id = self._find_max_value_in_column()
                 self._handle_job_error(job)
-                return max_id
+                if self.max_id_key:
+                    return self._find_max_value_in_column()
 
     def execute_complete(self, context: Context, event: dict[str, Any]):
         """
@@ -486,7 +478,6 @@ class GCSToBigQueryOperator(BaseOperator):
                 f"SELECT MAX({self.max_id_key}) AS max_value "
                 f"FROM {self.destination_project_dataset_table}"
             )
-
             self.configuration = {
                 "query": {
                     "query": select_command,
@@ -494,8 +485,17 @@ class GCSToBigQueryOperator(BaseOperator):
                     "schemaUpdateOptions": [],
                 }
             }
-            job_id = hook.insert_job(configuration=self.configuration, project_id=hook.project_id)
-            rows = list(hook.get_job(job_id=job_id, location=self.location).result())
+            try:
+                job_id = hook.insert_job(configuration=self.configuration, project_id=hook.project_id)
+                rows = list(hook.get_job(job_id=job_id, location=self.location).result())
+            except BadRequest as e:
+                if "Unrecognized name:" in e.message:
+                    raise AirflowException(
+                        f"Could not determine MAX value in column {self.max_id_key} "
+                        f"since the default value of 'string_field_n' was set by BQ"
+                    )
+                else:
+                    raise AirflowException(e.message)
             if rows:
                 for row in rows:
                     max_id = row[0] if row[0] else 0
@@ -509,53 +509,234 @@ class GCSToBigQueryOperator(BaseOperator):
             else:
                 raise RuntimeError(f"The {select_command} returned no rows!")
 
-    def _check_schema_fields(self, table_resource):
-        """
-        Helper method to detect schema fields if they were not specified by user and autodetect=True.
-        If source_objects were passed, method reads the second row in CSV file. If there is at least one digit
-        table_resurce is returned without changes so that BigQuery can determine schema_fields in the
-        next step.
-        If there are only characters, the first row with fields is used to construct schema_fields argument
-        with type 'STRING'. Table_resource is updated with new schema_fileds key and returned back to operator
-        :param table_resource: Configuration or table_resource dictionary
-        :return: table_resource: Updated table_resource dict with schema_fields
-        """
-        if not self.autodetect and not self.schema_fields:
-            raise RuntimeError(
-                "Table schema was not found. Set autodetect=True to "
-                "automatically set schema fields from source objects or pass "
-                "schema_fields explicitly"
-            )
-        elif not self.schema_fields:
-            for source_object in self.source_objects:
-                gcs_hook = GCSHook(
-                    gcp_conn_id=self.gcp_conn_id,
-                    delegate_to=self.delegate_to,
-                    impersonation_chain=self.impersonation_chain,
-                )
-                blob = gcs_hook.download(
-                    bucket_name=self.schema_object_bucket,
-                    object_name=source_object,
-                )
-                fields, values = [item.split(",") for item in blob.decode("utf-8").splitlines()][:2]
-                import re
+    def _create_empty_table(self):
+        self.project_id, dataset_id, table_id = self.hook.split_tablename(
+            table_input=self.destination_project_dataset_table,
+            default_project_id=self.project_id or self.hook.project_id,
+        )
 
-                if any(re.match(r"[\d\-\\.]+$", value) for value in values):
-                    return table_resource
-                else:
-                    schema_fields = []
-                    for field in fields:
-                        schema_fields.append({"name": field, "type": "STRING", "mode": "NULLABLE"})
-                    self.schema_fields = schema_fields
-                    if self.external_table:
-                        table_resource["externalDataConfiguration"]["csvOptions"]["skipLeadingRows"] = 1
-                    elif not self.external_table:
-                        table_resource["load"]["skipLeadingRows"] = 1
-        if self.external_table:
-            table_resource["schema"] = {"fields": self.schema_fields}
-        elif not self.external_table:
-            table_resource["load"]["schema"] = {"fields": self.schema_fields}
-        return table_resource
+        external_config_api_repr = {
+            "autodetect": self.autodetect,
+            "sourceFormat": self.source_format,
+            "sourceUris": self.source_uris,
+            "compression": self.compression.upper(),
+            "ignoreUnknownValues": self.ignore_unknown_values,
+        }
+        # if following fields are not specified in src_fmt_configs,
+        # honor the top-level params for backward-compatibility
+        backward_compatibility_configs = {
+            "skipLeadingRows": self.skip_leading_rows,
+            "fieldDelimiter": self.field_delimiter,
+            "quote": self.quote_character,
+            "allowQuotedNewlines": self.allow_quoted_newlines,
+            "allowJaggedRows": self.allow_jagged_rows,
+            "encoding": self.encoding,
+        }
+        src_fmt_to_param_mapping = {"CSV": "csvOptions", "GOOGLE_SHEETS": "googleSheetsOptions"}
+        src_fmt_to_configs_mapping = {
+            "csvOptions": [
+                "allowJaggedRows",
+                "allowQuotedNewlines",
+                "fieldDelimiter",
+                "skipLeadingRows",
+                "quote",
+                "encoding",
+            ],
+            "googleSheetsOptions": ["skipLeadingRows"],
+        }
+        if self.source_format in src_fmt_to_param_mapping.keys():
+            valid_configs = src_fmt_to_configs_mapping[src_fmt_to_param_mapping[self.source_format]]
+            self.src_fmt_configs = self._validate_src_fmt_configs(
+                self.source_format, self.src_fmt_configs, valid_configs, backward_compatibility_configs
+            )
+            external_config_api_repr[src_fmt_to_param_mapping[self.source_format]] = self.src_fmt_configs
+
+        external_config = ExternalConfig.from_api_repr(external_config_api_repr)
+        if self.schema_fields:
+            external_config.schema = [SchemaField.from_api_repr(f) for f in self.schema_fields]
+        if self.max_bad_records:
+            external_config.max_bad_records = self.max_bad_records
+
+        # build table definition
+        table = Table(
+            table_ref=TableReference.from_string(self.destination_project_dataset_table, self.project_id)
+        )
+        table.external_data_configuration = external_config
+        if self.labels:
+            table.labels = self.labels
+
+        if self.description:
+            table.description = self.description
+
+        if self.encryption_configuration:
+            table.encryption_configuration = EncryptionConfiguration.from_api_repr(
+                self.encryption_configuration
+            )
+        table_obj_api_repr = table.to_api_repr()
+
+        self.log.info("Creating external table: %s", self.destination_project_dataset_table)
+        self.hook.create_empty_table(
+            table_resource=table_obj_api_repr,
+            project_id=self.project_id,
+            location=self.location,
+            exists_ok=True,
+        )
+        self.log.info("External table created successfully: %s", self.destination_project_dataset_table)
+        return table_obj_api_repr
+
+    def _use_existing_table(self):
+        self.project_id, destination_dataset, destination_table = self.hook.split_tablename(
+            table_input=self.destination_project_dataset_table,
+            default_project_id=self.project_id or self.hook.project_id,
+            var_name="destination_project_dataset_table",
+        )
+
+        # bigquery also allows you to define how you want a table's schema to change
+        # as a side effect of a load
+        # for more details:
+        # https://cloud.google.com/bigquery/docs/reference/rest/v2/jobs#configuration.load.schemaUpdateOptions
+        allowed_schema_update_options = ["ALLOW_FIELD_ADDITION", "ALLOW_FIELD_RELAXATION"]
+        if not set(allowed_schema_update_options).issuperset(set(self.schema_update_options)):
+            raise ValueError(
+                f"{self.schema_update_options} contains invalid schema update options. "
+                f"Please only use one or more of the following options: {allowed_schema_update_options}"
+            )
+
+        self.configuration = {
+            "load": {
+                "autodetect": self.autodetect,
+                "createDisposition": self.create_disposition,
+                "destinationTable": {
+                    "projectId": self.project_id,
+                    "datasetId": destination_dataset,
+                    "tableId": destination_table,
+                },
+                "sourceFormat": self.source_format,
+                "sourceUris": self.source_uris,
+                "writeDisposition": self.write_disposition,
+                "ignoreUnknownValues": self.ignore_unknown_values,
+            },
+        }
+        self.time_partitioning = self._cleanse_time_partitioning(
+            self.destination_project_dataset_table, self.time_partitioning
+        )
+        if self.time_partitioning:
+            self.configuration["load"].update({"timePartitioning": self.time_partitioning})
+
+        if self.cluster_fields:
+            self.configuration["load"].update({"clustering": {"fields": self.cluster_fields}})
+
+        if self.schema_fields:
+            self.configuration["load"]["schema"] = {"fields": self.schema_fields}
+
+        if self.schema_update_options:
+            if self.write_disposition not in ["WRITE_APPEND", "WRITE_TRUNCATE"]:
+                raise ValueError(
+                    "schema_update_options is only "
+                    "allowed if write_disposition is "
+                    "'WRITE_APPEND' or 'WRITE_TRUNCATE'."
+                )
+            else:
+                # To provide backward compatibility
+                self.schema_update_options = list(self.schema_update_options or [])
+                self.log.info("Adding experimental 'schemaUpdateOptions': %s", self.schema_update_options)
+                self.configuration["load"]["schemaUpdateOptions"] = self.schema_update_options
+
+        if self.max_bad_records:
+            self.configuration["load"]["maxBadRecords"] = self.max_bad_records
+
+        if self.encryption_configuration:
+            self.configuration["load"]["destinationEncryptionConfiguration"] = self.encryption_configuration
+
+        if self.labels or self.description:
+            self.configuration["load"].update({"destinationTableProperties": {}})
+            if self.labels:
+                self.configuration["load"]["destinationTableProperties"]["labels"] = self.labels
+            if self.description:
+                self.configuration["load"]["destinationTableProperties"]["description"] = self.description
+
+        src_fmt_to_configs_mapping = {
+            "CSV": [
+                "allowJaggedRows",
+                "allowQuotedNewlines",
+                "autodetect",
+                "fieldDelimiter",
+                "skipLeadingRows",
+                "ignoreUnknownValues",
+                "nullMarker",
+                "quote",
+                "encoding",
+            ],
+            "DATASTORE_BACKUP": ["projectionFields"],
+            "NEWLINE_DELIMITED_JSON": ["autodetect", "ignoreUnknownValues"],
+            "PARQUET": ["autodetect", "ignoreUnknownValues"],
+            "AVRO": ["useAvroLogicalTypes"],
+        }
+
+        valid_configs = src_fmt_to_configs_mapping[self.source_format]
+
+        # if following fields are not specified in src_fmt_configs,
+        # honor the top-level params for backward-compatibility
+        backward_compatibility_configs = {
+            "skipLeadingRows": self.skip_leading_rows,
+            "fieldDelimiter": self.field_delimiter,
+            "ignoreUnknownValues": self.ignore_unknown_values,
+            "quote": self.quote_character,
+            "allowQuotedNewlines": self.allow_quoted_newlines,
+            "encoding": self.encoding,
+        }
+
+        self.src_fmt_configs = self._validate_src_fmt_configs(
+            self.source_format, self.src_fmt_configs, valid_configs, backward_compatibility_configs
+        )
+
+        self.configuration["load"].update(self.src_fmt_configs)
+
+        if self.allow_jagged_rows:
+            self.configuration["load"]["allowJaggedRows"] = self.allow_jagged_rows
+        return self.configuration
+
+    def _validate_src_fmt_configs(
+        self,
+        source_format: str,
+        src_fmt_configs: dict,
+        valid_configs: list[str],
+        backward_compatibility_configs: dict | None = None,
+    ) -> dict:
+        """
+        Validates the given src_fmt_configs against a valid configuration for the source format.
+        Adds the backward compatibility config to the src_fmt_configs.
+
+        :param source_format: File format to export.
+        :param src_fmt_configs: Configure optional fields specific to the source format.
+        :param valid_configs: Valid configuration specific to the source format
+        :param backward_compatibility_configs: The top-level params for backward-compatibility
+        """
+        if backward_compatibility_configs is None:
+            backward_compatibility_configs = {}
+
+        for k, v in backward_compatibility_configs.items():
+            if k not in src_fmt_configs and k in valid_configs:
+                src_fmt_configs[k] = v
+
+        for k, v in src_fmt_configs.items():
+            if k not in valid_configs:
+                raise ValueError(f"{k} is not a valid src_fmt_configs for type {source_format}.")
+
+        return src_fmt_configs
+
+    def _cleanse_time_partitioning(
+        self, destination_dataset_table: str | None, time_partitioning_in: dict | None
+    ) -> dict:  # if it is a partitioned table ($ is in the table name) add partition load option
+
+        if time_partitioning_in is None:
+            time_partitioning_in = {}
+
+        time_partitioning_out = {}
+        if destination_dataset_table and "$" in destination_dataset_table:
+            time_partitioning_out["type"] = "DAY"
+        time_partitioning_out.update(time_partitioning_in)
+        return time_partitioning_out
 
     def on_kill(self) -> None:
         if self.job_id and self.cancel_on_kill:
