@@ -16,14 +16,18 @@
 # specific language governing permissions and limitations
 # under the License.
 """Base operator for SQL to GCS operators."""
+from __future__ import annotations
+
 import abc
+import csv
 import json
+import os
+import warnings
 from tempfile import NamedTemporaryFile
-from typing import TYPE_CHECKING, Dict, Optional, Sequence, Union
+from typing import TYPE_CHECKING, Sequence
 
 import pyarrow as pa
 import pyarrow.parquet as pq
-import unicodecsv as csv
 
 from airflow.models import BaseOperator
 from airflow.providers.google.cloud.hooks.gcs import GCSHook
@@ -51,6 +55,8 @@ class BaseSQLToGCSOperator(BaseOperator):
         file size of the splits. Check https://cloud.google.com/storage/quotas
         to see the maximum allowed file size for a single object.
     :param export_format: Desired format of files to be exported. (json, csv or parquet)
+    :param stringify_dict: Whether to dump Dictionary type objects
+        (such as JSON columns) as a string. Applies only to CSV/JSON export format.
     :param field_delimiter: The delimiter to be used for CSV files.
     :param null_marker: The null marker to be used for CSV files.
     :param gzip: Option to compress file for upload (does not apply to schemas).
@@ -73,20 +79,28 @@ class BaseSQLToGCSOperator(BaseOperator):
         account from the list granting this role to the originating account (templated).
     :param upload_metadata: whether to upload the row count metadata as blob metadata
     :param exclude_columns: set of columns to exclude from transmission
+    :param partition_columns: list of columns to use for file partitioning. In order to use
+        this parameter, you must sort your dataset by partition_columns. Do this by
+        passing an ORDER BY clause to the sql query. Files are uploaded to GCS as objects
+        with a hive style partitioning directory structure (templated).
+    :param write_on_empty: Optional parameter to specify whether to write a file if the
+        export does not return any rows. Default is False so we will not write a file
+        if the export returns no rows.
     """
 
     template_fields: Sequence[str] = (
-        'sql',
-        'bucket',
-        'filename',
-        'schema_filename',
-        'schema',
-        'parameters',
-        'impersonation_chain',
+        "sql",
+        "bucket",
+        "filename",
+        "schema_filename",
+        "schema",
+        "parameters",
+        "impersonation_chain",
+        "partition_columns",
     )
-    template_ext: Sequence[str] = ('.sql',)
-    template_fields_renderers = {'sql': 'sql'}
-    ui_color = '#a0e08c'
+    template_ext: Sequence[str] = (".sql",)
+    template_fields_renderers = {"sql": "sql"}
+    ui_color = "#a0e08c"
 
     def __init__(
         self,
@@ -94,19 +108,22 @@ class BaseSQLToGCSOperator(BaseOperator):
         sql: str,
         bucket: str,
         filename: str,
-        schema_filename: Optional[str] = None,
+        schema_filename: str | None = None,
         approx_max_file_size_bytes: int = 1900000000,
-        export_format: str = 'json',
-        field_delimiter: str = ',',
-        null_marker: Optional[str] = None,
+        export_format: str = "json",
+        stringify_dict: bool = False,
+        field_delimiter: str = ",",
+        null_marker: str | None = None,
         gzip: bool = False,
-        schema: Optional[Union[str, list]] = None,
-        parameters: Optional[dict] = None,
-        gcp_conn_id: str = 'google_cloud_default',
-        delegate_to: Optional[str] = None,
-        impersonation_chain: Optional[Union[str, Sequence[str]]] = None,
+        schema: str | list | None = None,
+        parameters: dict | None = None,
+        gcp_conn_id: str = "google_cloud_default",
+        delegate_to: str | None = None,
+        impersonation_chain: str | Sequence[str] | None = None,
         upload_metadata: bool = False,
-        exclude_columns=None,
+        exclude_columns: set | None = None,
+        partition_columns: list | None = None,
+        write_on_empty: bool = False,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -119,74 +136,89 @@ class BaseSQLToGCSOperator(BaseOperator):
         self.schema_filename = schema_filename
         self.approx_max_file_size_bytes = approx_max_file_size_bytes
         self.export_format = export_format.lower()
+        self.stringify_dict = stringify_dict
         self.field_delimiter = field_delimiter
         self.null_marker = null_marker
         self.gzip = gzip
         self.schema = schema
         self.parameters = parameters
         self.gcp_conn_id = gcp_conn_id
+        if delegate_to:
+            warnings.warn(
+                "'delegate_to' parameter is deprecated, please use 'impersonation_chain'", DeprecationWarning
+            )
         self.delegate_to = delegate_to
         self.impersonation_chain = impersonation_chain
         self.upload_metadata = upload_metadata
         self.exclude_columns = exclude_columns
+        self.partition_columns = partition_columns
+        self.write_on_empty = write_on_empty
+
+    def execute(self, context: Context):
+        if self.partition_columns:
+            self.log.info(
+                f"Found partition columns: {','.join(self.partition_columns)}. "
+                "Assuming the SQL statement is properly sorted by these columns in "
+                "ascending or descending order."
+            )
 
-    def execute(self, context: 'Context'):
         self.log.info("Executing query")
         cursor = self.query()
 
         # If a schema is set, create a BQ schema JSON file.
         if self.schema_filename:
-            self.log.info('Writing local schema file')
+            self.log.info("Writing local schema file")
             schema_file = self._write_local_schema_file(cursor)
 
             # Flush file before uploading
-            schema_file['file_handle'].flush()
+            schema_file["file_handle"].flush()
 
-            self.log.info('Uploading schema file to GCS.')
+            self.log.info("Uploading schema file to GCS.")
             self._upload_to_gcs(schema_file)
-            schema_file['file_handle'].close()
+            schema_file["file_handle"].close()
 
         counter = 0
         files = []
         total_row_count = 0
         total_files = 0
-        self.log.info('Writing local data files')
+        self.log.info("Writing local data files")
         for file_to_upload in self._write_local_data_files(cursor):
-            # Flush file before uploading
-            file_to_upload['file_handle'].flush()
 
-            self.log.info('Uploading chunk file #%d to GCS.', counter)
+            # Flush file before uploading
+            file_to_upload["file_handle"].flush()
+
+            self.log.info("Uploading chunk file #%d to GCS.", counter)
             self._upload_to_gcs(file_to_upload)
 
-            self.log.info('Removing local file')
-            file_to_upload['file_handle'].close()
+            self.log.info("Removing local file")
+            file_to_upload["file_handle"].close()
 
             # Metadata to be outputted to Xcom
-            total_row_count += file_to_upload['file_row_count']
+            total_row_count += file_to_upload["file_row_count"]
             total_files += 1
             files.append(
                 {
-                    'file_name': file_to_upload['file_name'],
-                    'file_mime_type': file_to_upload['file_mime_type'],
-                    'file_row_count': file_to_upload['file_row_count'],
+                    "file_name": file_to_upload["file_name"],
+                    "file_mime_type": file_to_upload["file_mime_type"],
+                    "file_row_count": file_to_upload["file_row_count"],
                 }
             )
 
             counter += 1
 
         file_meta = {
-            'bucket': self.bucket,
-            'total_row_count': total_row_count,
-            'total_files': total_files,
-            'files': files,
+            "bucket": self.bucket,
+            "total_row_count": total_row_count,
+            "total_files": total_files,
+            "files": files,
         }
 
         return file_meta
 
-    def convert_types(self, schema, col_type_dict, row, stringify_dict=False) -> list:
+    def convert_types(self, schema, col_type_dict, row) -> list:
         """Convert values from DBAPI to output-friendly formats."""
         return [
-            self.convert_type(value, col_type_dict.get(name), stringify_dict=stringify_dict)
+            self.convert_type(value, col_type_dict.get(name), stringify_dict=self.stringify_dict)
             for name, value in zip(schema, row)
         ]
 
@@ -198,42 +230,62 @@ class BaseSQLToGCSOperator(BaseOperator):
             names in GCS, and values are file handles to local files that
             contain the data for the GCS objects.
         """
-        import os
-
         org_schema = list(map(lambda schema_tuple: schema_tuple[0], cursor.description))
         schema = [column for column in org_schema if column not in self.exclude_columns]
 
         col_type_dict = self._get_col_type_dict()
         file_no = 0
+        file_mime_type = self._get_file_mime_type()
+        file_to_upload, tmp_file_handle = self._get_file_to_upload(file_mime_type, file_no)
 
-        tmp_file_handle = NamedTemporaryFile(delete=True)
-        if self.export_format == 'csv':
-            file_mime_type = 'text/csv'
-        elif self.export_format == 'parquet':
-            file_mime_type = 'application/octet-stream'
-        else:
-            file_mime_type = 'application/json'
-        file_to_upload = {
-            'file_name': self.filename.format(file_no),
-            'file_handle': tmp_file_handle,
-            'file_mime_type': file_mime_type,
-            'file_row_count': 0,
-        }
-
-        if self.export_format == 'csv':
+        if self.export_format == "csv":
             csv_writer = self._configure_csv_file(tmp_file_handle, schema)
-        if self.export_format == 'parquet':
+        if self.export_format == "parquet":
             parquet_schema = self._convert_parquet_schema(cursor)
             parquet_writer = self._configure_parquet_file(tmp_file_handle, parquet_schema)
 
+        prev_partition_values = None
+        curr_partition_values = None
         for row in cursor:
-            file_to_upload['file_row_count'] += 1
-            if self.export_format == 'csv':
+            if self.partition_columns:
+                row_dict = dict(zip(schema, row))
+                curr_partition_values = tuple(
+                    [row_dict.get(partition_column, "") for partition_column in self.partition_columns]
+                )
+
+                if prev_partition_values is None:
+                    # We haven't set prev_partition_values before. Set to current
+                    prev_partition_values = curr_partition_values
+
+                elif prev_partition_values != curr_partition_values:
+                    # If the partition values differ, write the current local file out
+                    # Yield first before we write the current record
+                    file_no += 1
+
+                    if self.export_format == "parquet":
+                        parquet_writer.close()
+
+                    file_to_upload["partition_values"] = prev_partition_values
+                    yield file_to_upload
+                    file_to_upload, tmp_file_handle = self._get_file_to_upload(file_mime_type, file_no)
+                    if self.export_format == "csv":
+                        csv_writer = self._configure_csv_file(tmp_file_handle, schema)
+                    if self.export_format == "parquet":
+                        parquet_writer = self._configure_parquet_file(tmp_file_handle, parquet_schema)
+
+                    # Reset previous to current after writing out the file
+                    prev_partition_values = curr_partition_values
+
+            # Incrementing file_row_count after partition yield ensures all rows are written
+            file_to_upload["file_row_count"] += 1
+
+            # Proceed to write the row to the localfile
+            if self.export_format == "csv":
                 row = self.convert_types(schema, col_type_dict, row)
                 if self.null_marker is not None:
                     row = [value if value is not None else self.null_marker for value in row]
                 csv_writer.writerow(row)
-            elif self.export_format == 'parquet':
+            elif self.export_format == "parquet":
                 row = self.convert_types(schema, col_type_dict, row)
                 if self.null_marker is not None:
                     row = [value if value is not None else self.null_marker for value in row]
@@ -241,15 +293,13 @@ class BaseSQLToGCSOperator(BaseOperator):
                 tbl = pa.Table.from_pydict(row_pydic, parquet_schema)
                 parquet_writer.write_table(tbl)
             else:
-                row = self.convert_types(schema, col_type_dict, row, stringify_dict=False)
+                row = self.convert_types(schema, col_type_dict, row)
                 row_dict = dict(zip(schema, row))
 
-                tmp_file_handle.write(
-                    json.dumps(row_dict, sort_keys=True, ensure_ascii=False).encode("utf-8")
-                )
+                json.dump(row_dict, tmp_file_handle, sort_keys=True, ensure_ascii=False)
 
                 # Append newline to make dumps BigQuery compatible.
-                tmp_file_handle.write(b'\n')
+                tmp_file_handle.write("\n")
 
             # Stop if the file exceeds the file size limit.
             fppos = tmp_file_handle.tell()
@@ -260,31 +310,52 @@ class BaseSQLToGCSOperator(BaseOperator):
             if file_size >= self.approx_max_file_size_bytes:
                 file_no += 1
 
-                if self.export_format == 'parquet':
+                if self.export_format == "parquet":
                     parquet_writer.close()
+
+                file_to_upload["partition_values"] = curr_partition_values
                 yield file_to_upload
-                tmp_file_handle = NamedTemporaryFile(delete=True)
-                file_to_upload = {
-                    'file_name': self.filename.format(file_no),
-                    'file_handle': tmp_file_handle,
-                    'file_mime_type': file_mime_type,
-                    'file_row_count': 0,
-                }
-                if self.export_format == 'csv':
+                file_to_upload, tmp_file_handle = self._get_file_to_upload(file_mime_type, file_no)
+                if self.export_format == "csv":
                     csv_writer = self._configure_csv_file(tmp_file_handle, schema)
-                if self.export_format == 'parquet':
+                if self.export_format == "parquet":
                     parquet_writer = self._configure_parquet_file(tmp_file_handle, parquet_schema)
-        if self.export_format == 'parquet':
+
+        if self.export_format == "parquet":
             parquet_writer.close()
         # Last file may have 0 rows, don't yield if empty
-        if file_to_upload['file_row_count'] > 0:
+        # However, if it is the first file and self.write_on_empty is True, then yield to write an empty file
+        if file_to_upload["file_row_count"] > 0 or (file_no == 0 and self.write_on_empty):
+            file_to_upload["partition_values"] = curr_partition_values
             yield file_to_upload
 
+    def _get_file_to_upload(self, file_mime_type, file_no):
+        """Returns a dictionary that represents the file to upload."""
+        tmp_file_handle = NamedTemporaryFile(mode="w", encoding="utf-8", delete=True)
+        return (
+            {
+                "file_name": self.filename.format(file_no),
+                "file_handle": tmp_file_handle,
+                "file_mime_type": file_mime_type,
+                "file_row_count": 0,
+            },
+            tmp_file_handle,
+        )
+
+    def _get_file_mime_type(self):
+        if self.export_format == "csv":
+            file_mime_type = "text/csv"
+        elif self.export_format == "parquet":
+            file_mime_type = "application/octet-stream"
+        else:
+            file_mime_type = "application/json"
+        return file_mime_type
+
     def _configure_csv_file(self, file_handle, schema):
         """Configure a csv writer with the file_handle and write schema
         as headers for the new file.
         """
-        csv_writer = csv.writer(file_handle, encoding='utf-8', delimiter=self.field_delimiter)
+        csv_writer = csv.writer(file_handle, delimiter=self.field_delimiter)
         csv_writer.writerow(schema)
         return csv_writer
 
@@ -294,21 +365,21 @@ class BaseSQLToGCSOperator(BaseOperator):
 
     def _convert_parquet_schema(self, cursor):
         type_map = {
-            'INTEGER': pa.int64(),
-            'FLOAT': pa.float64(),
-            'NUMERIC': pa.float64(),
-            'BIGNUMERIC': pa.float64(),
-            'BOOL': pa.bool_(),
-            'STRING': pa.string(),
-            'BYTES': pa.binary(),
-            'DATE': pa.date32(),
-            'DATETIME': pa.date64(),
-            'TIMESTAMP': pa.timestamp('s'),
+            "INTEGER": pa.int64(),
+            "FLOAT": pa.float64(),
+            "NUMERIC": pa.float64(),
+            "BIGNUMERIC": pa.float64(),
+            "BOOL": pa.bool_(),
+            "STRING": pa.string(),
+            "BYTES": pa.binary(),
+            "DATE": pa.date32(),
+            "DATETIME": pa.date64(),
+            "TIMESTAMP": pa.timestamp("s"),
         }
 
         columns = [field[0] for field in cursor.description]
         bq_fields = [self.field_to_bigquery(field) for field in cursor.description]
-        bq_types = [bq_field.get('type') if bq_field is not None else None for bq_field in bq_fields]
+        bq_types = [bq_field.get("type") if bq_field is not None else None for bq_field in bq_fields]
         pq_types = [type_map.get(bq_type, pa.string()) for bq_type in bq_types]
         parquet_schema = pa.schema(zip(columns, pq_types))
         return parquet_schema
@@ -318,7 +389,7 @@ class BaseSQLToGCSOperator(BaseOperator):
         """Execute DBAPI query."""
 
     @abc.abstractmethod
-    def field_to_bigquery(self, field) -> Dict[str, str]:
+    def field_to_bigquery(self, field) -> dict[str, str]:
         """Convert a DBAPI field to BigQuery schema format."""
 
     @abc.abstractmethod
@@ -333,16 +404,16 @@ class BaseSQLToGCSOperator(BaseOperator):
         elif isinstance(self.schema, list):
             schema = self.schema
         elif self.schema is not None:
-            self.log.warning('Using default schema due to unexpected type. Should be a string or list.')
+            self.log.warning("Using default schema due to unexpected type. Should be a string or list.")
 
         col_type_dict = {}
         try:
-            col_type_dict = {col['name']: col['type'] for col in schema}
+            col_type_dict = {col["name"]: col["type"] for col in schema}
         except KeyError:
             self.log.warning(
-                'Using default schema due to missing name or type. Please '
-                'refer to: https://cloud.google.com/bigquery/docs/schemas'
-                '#specifying_a_json_schema_file'
+                "Using default schema due to missing name or type. Please "
+                "refer to: https://cloud.google.com/bigquery/docs/schemas"
+                "#specifying_a_json_schema_file"
             )
         return col_type_dict
 
@@ -370,15 +441,15 @@ class BaseSQLToGCSOperator(BaseOperator):
         if isinstance(schema, list):
             schema = json.dumps(schema, sort_keys=True)
 
-        self.log.info('Using schema for %s', self.schema_filename)
+        self.log.info("Using schema for %s", self.schema_filename)
         self.log.debug("Current schema: %s", schema)
 
-        tmp_schema_file_handle = NamedTemporaryFile(delete=True)
-        tmp_schema_file_handle.write(schema.encode('utf-8'))
+        tmp_schema_file_handle = NamedTemporaryFile(mode="w", encoding="utf-8", delete=True)
+        tmp_schema_file_handle.write(schema)
         schema_file_to_upload = {
-            'file_name': self.schema_filename,
-            'file_handle': tmp_schema_file_handle,
-            'file_mime_type': 'application/json',
+            "file_name": self.schema_filename,
+            "file_handle": tmp_schema_file_handle,
+            "file_mime_type": "application/json",
         }
         return schema_file_to_upload
 
@@ -389,16 +460,26 @@ class BaseSQLToGCSOperator(BaseOperator):
             delegate_to=self.delegate_to,
             impersonation_chain=self.impersonation_chain,
         )
-        is_data_file = file_to_upload.get('file_name') != self.schema_filename
+        is_data_file = file_to_upload.get("file_name") != self.schema_filename
         metadata = None
         if is_data_file and self.upload_metadata:
-            metadata = {'row_count': file_to_upload['file_row_count']}
+            metadata = {"row_count": file_to_upload["file_row_count"]}
+
+        object_name = file_to_upload.get("file_name")
+        if is_data_file and self.partition_columns:
+            # Add partition column values to object_name
+            partition_values = file_to_upload.get("partition_values")
+            head_path, tail_path = os.path.split(object_name)
+            partition_subprefix = [
+                f"{col}={val}" for col, val in zip(self.partition_columns, partition_values)
+            ]
+            object_name = os.path.join(head_path, *partition_subprefix, tail_path)
 
         hook.upload(
             self.bucket,
-            file_to_upload.get('file_name'),
-            file_to_upload.get('file_handle').name,
-            mime_type=file_to_upload.get('file_mime_type'),
+            object_name,
+            file_to_upload.get("file_handle").name,
+            mime_type=file_to_upload.get("file_mime_type"),
             gzip=self.gzip if is_data_file else False,
             metadata=metadata,
         )
