@@ -29,6 +29,7 @@ import dill
 from airflow import DAG
 from airflow.exceptions import AirflowException
 from airflow.operators.python import PythonOperator
+from airflow.providers.apache.beam.hooks.beam import BeamRunnerType
 from airflow.providers.apache.beam.operators.beam import BeamRunPythonPipelineOperator
 from airflow.providers.google.cloud.hooks.gcs import GCSHook
 from airflow.providers.google.cloud.operators.mlengine import MLEngineStartBatchPredictionJobOperator
@@ -227,6 +228,7 @@ def create_evaluate_ops(
     metric_fn_encoded = base64.b64encode(dill.dumps(metric_fn, recurse=True)).decode()
     evaluate_summary = BeamRunPythonPipelineOperator(
         task_id=(task_prefix + "-summary"),
+        runner=BeamRunnerType.DataflowRunner,
         py_file=os.path.join(os.path.dirname(__file__), "mlengine_prediction_summary.py"),
         default_pipeline_options=dataflow_options,
         pipeline_options={
@@ -235,7 +237,7 @@ def create_evaluate_ops(
             "metric_keys": ",".join(metric_keys),
         },
         py_interpreter=py_interpreter,
-        py_requirements=["apache-beam[gcp]>=2.14.0"],
+        py_requirements=["apache-beam[gcp]>=2.46.0"],
         dag=dag,
     )
     evaluate_summary.set_upstream(evaluate_prediction)
