@@ -16,7 +16,9 @@
 # specific language governing permissions and limitations
 # under the License.
 """Hook for sending or receiving data from PagerDuty as well as creating PagerDuty incidents."""
-from typing import Any, Dict, List, Optional
+from __future__ import annotations
+
+from typing import Any
 
 import pdpyras
 
@@ -41,17 +43,17 @@ class PagerdutyEventsHook(BaseHook):
     hook_name = "Pagerduty Events"
 
     @staticmethod
-    def get_ui_field_behaviour() -> Dict[str, Any]:
+    def get_ui_field_behaviour() -> dict[str, Any]:
         """Returns custom field behaviour"""
         return {
-            "hidden_fields": ['port', 'login', 'schema', 'host', 'extra'],
+            "hidden_fields": ["port", "login", "schema", "host", "extra"],
             "relabeling": {
-                'password': 'Pagerduty Integration key',
+                "password": "Pagerduty Integration key",
             },
         }
 
     def __init__(
-        self, integration_key: Optional[str] = None, pagerduty_events_conn_id: Optional[str] = None
+        self, integration_key: str | None = None, pagerduty_events_conn_id: str | None = None
     ) -> None:
         super().__init__()
         self.integration_key = None
@@ -66,23 +68,23 @@ class PagerdutyEventsHook(BaseHook):
 
         if self.integration_key is None:
             raise AirflowException(
-                'Cannot get token: No valid integration key nor pagerduty_events_conn_id supplied.'
+                "Cannot get token: No valid integration key nor pagerduty_events_conn_id supplied."
             )
 
     def create_event(
         self,
         summary: str,
         severity: str,
-        source: str = 'airflow',
-        action: str = 'trigger',
-        dedup_key: Optional[str] = None,
-        custom_details: Optional[Any] = None,
-        group: Optional[str] = None,
-        component: Optional[str] = None,
-        class_type: Optional[str] = None,
-        images: Optional[List[Any]] = None,
-        links: Optional[List[Any]] = None,
-    ) -> Dict:
+        source: str = "airflow",
+        action: str = "trigger",
+        dedup_key: str | None = None,
+        custom_details: Any | None = None,
+        group: str | None = None,
+        component: str | None = None,
+        class_type: str | None = None,
+        images: list[Any] | None = None,
+        links: list[Any] | None = None,
+    ) -> dict:
         """
         Create event for service integration.
 
@@ -110,7 +112,6 @@ class PagerdutyEventsHook(BaseHook):
             `text`: [Optional] Plain text that describes the purpose of the link, and can be used as the
             link's text.
         :return: PagerDuty Events API v2 response.
-        :rtype: dict
         """
         payload = {
             "summary": summary,
@@ -126,7 +127,7 @@ class PagerdutyEventsHook(BaseHook):
         if class_type:
             payload["class"] = class_type
 
-        actions = ('trigger', 'acknowledge', 'resolve')
+        actions = ("trigger", "acknowledge", "resolve")
         if action not in actions:
             raise ValueError(f"Event action must be one of: {', '.join(actions)}")
         data = {
@@ -135,7 +136,7 @@ class PagerdutyEventsHook(BaseHook):
         }
         if dedup_key:
             data["dedup_key"] = dedup_key
-        elif action != 'trigger':
+        elif action != "trigger":
             raise ValueError(
                 f"The dedup_key property is required for event_action={action} events, "
                 f"and it must be a string."
