@@ -24,16 +24,17 @@ import logging
 import os
 import sys
 import warnings
-from typing import TYPE_CHECKING, Callable
+from typing import TYPE_CHECKING, Any, Callable
 
 import pendulum
+import pluggy
 import sqlalchemy
 from sqlalchemy import create_engine, exc
 from sqlalchemy.engine import Engine
-from sqlalchemy.orm import scoped_session, sessionmaker
-from sqlalchemy.orm.session import Session as SASession
+from sqlalchemy.orm import Session as SASession, scoped_session, sessionmaker
 from sqlalchemy.pool import NullPool
 
+from airflow import policies
 from airflow.configuration import AIRFLOW_HOME, WEBSERVER_CONFIG, conf  # NOQA F401
 from airflow.exceptions import RemovedInAirflow3Warning
 from airflow.executors import executor_constants
@@ -117,7 +118,7 @@ def _get_rich_console(file):
 
 
 def custom_show_warning(message, category, filename, lineno, file=None, line=None):
-    """Custom function to print rich and visible warnings"""
+    """Custom function to print rich and visible warnings."""
     # Delay imports until we need it
     from rich.markup import escape
 
@@ -142,107 +143,51 @@ def replace_showwarning(replacement):
 original_show_warning = replace_showwarning(custom_show_warning)
 atexit.register(functools.partial(replace_showwarning, original_show_warning))
 
-
-def task_policy(task) -> None:
-    """
-    This policy setting allows altering tasks after they are loaded in
-    the DagBag. It allows administrator to rewire some task's parameters.
-    Alternatively you can raise ``AirflowClusterPolicyViolation`` exception
-    to stop DAG from being executed.
-
-    To define policy, add a ``airflow_local_settings`` module
-    to your PYTHONPATH that defines this ``task_policy`` function.
-
-    Here are a few examples of how this can be useful:
-
-    * You could enforce a specific queue (say the ``spark`` queue)
-        for tasks using the ``SparkOperator`` to make sure that these
-        tasks get wired to the right workers
-    * You could enforce a task timeout policy, making sure that no tasks run
-        for more than 48 hours
-
-    :param task: task to be mutated
-    """
+POLICY_PLUGIN_MANAGER: Any = None  # type: ignore
 
 
-def dag_policy(dag) -> None:
-    """
-    This policy setting allows altering DAGs after they are loaded in
-    the DagBag. It allows administrator to rewire some DAG's parameters.
-    Alternatively you can raise ``AirflowClusterPolicyViolation`` exception
-    to stop DAG from being executed.
+def task_policy(task):
+    return POLICY_PLUGIN_MANAGER.hook.task_policy(task=task)
 
-    To define policy, add a ``airflow_local_settings`` module
-    to your PYTHONPATH that defines this ``dag_policy`` function.
 
-    Here are a few examples of how this can be useful:
-
-    * You could enforce default user for DAGs
-    * Check if every DAG has configured tags
-
-    :param dag: dag to be mutated
-    """
+def dag_policy(dag):
+    return POLICY_PLUGIN_MANAGER.hook.dag_policy(dag=dag)
 
 
 def task_instance_mutation_hook(task_instance):
-    """
-    This setting allows altering task instances before they are queued by
-    the Airflow scheduler.
-
-    To define task_instance_mutation_hook, add a ``airflow_local_settings`` module
-    to your PYTHONPATH that defines this ``task_instance_mutation_hook`` function.
-
-    This could be used, for instance, to modify the task instance during retries.
-
-    :param task_instance: task instance to be mutated
-    """
+    return POLICY_PLUGIN_MANAGER.hook.task_instance_mutation_hook(task_instance=task_instance)
 
 
 task_instance_mutation_hook.is_noop = True  # type: ignore
 
 
 def pod_mutation_hook(pod):
-    """
-    This setting allows altering ``kubernetes.client.models.V1Pod`` object
-    before they are passed to the Kubernetes client for scheduling.
-
-    To define a pod mutation hook, add a ``airflow_local_settings`` module
-    to your PYTHONPATH that defines this ``pod_mutation_hook`` function.
-    It receives a ``Pod`` object and can alter it where needed.
-
-    This could be used, for instance, to add sidecar or init containers
-    to every worker pod launched by KubernetesExecutor or KubernetesPodOperator.
-    """
+    return POLICY_PLUGIN_MANAGER.hook.pod_mutation_hook(pod=pod)
 
 
 def get_airflow_context_vars(context):
-    """
-    This setting allows getting the airflow context vars, which are key value pairs.
-    They are then injected to default airflow context vars, which in the end are
-    available as environment variables when running tasks
-    dag_id, task_id, execution_date, dag_run_id, try_number are reserved keys.
-    To define it, add a ``airflow_local_settings`` module
-    to your PYTHONPATH that defines this ``get_airflow_context_vars`` function.
-
-    :param context: The context for the task_instance of interest.
-    """
-    return {}
+    return POLICY_PLUGIN_MANAGER.hook.get_airflow_context_vars(context=context)
 
 
-def get_dagbag_import_timeout(dag_file_path: str) -> int | float:
-    """
-    This setting allows for dynamic control of the DAG file parsing timeout based on the DAG file path.
+def get_dagbag_import_timeout(dag_file_path: str):
+    return POLICY_PLUGIN_MANAGER.hook.get_dagbag_import_timeout(dag_file_path=dag_file_path)
 
-    It is useful when there are a few DAG files requiring longer parsing times, while others do not.
-    You can control them separately instead of having one value for all DAG files.
 
-    If the return value is less than or equal to 0, it means no timeout during the DAG parsing.
-    """
-    return conf.getfloat("core", "DAGBAG_IMPORT_TIMEOUT")
+def configure_policy_plugin_manager():
+    global POLICY_PLUGIN_MANAGER
+
+    POLICY_PLUGIN_MANAGER = pluggy.PluginManager(policies.local_settings_hookspec.project_name)
+    POLICY_PLUGIN_MANAGER.add_hookspecs(policies)
+    POLICY_PLUGIN_MANAGER.register(policies.DefaultPolicy)
+
+
+def load_policy_plugins(pm: pluggy.PluginManager):
+    # We can't log duration etc  here, as logging hasn't yet been configured!
+    pm.load_setuptools_entrypoints("airflow.policy")
 
 
 def configure_vars():
-    """Configure Global Variables from airflow.cfg"""
+    """Configure Global Variables from airflow.cfg."""
     global SQL_ALCHEMY_CONN
     global DAGS_FOLDER
     global PLUGINS_FOLDER
@@ -261,7 +206,7 @@ def configure_vars():
 
 
 def configure_orm(disable_connection_pool=False, pool_class=None):
-    """Configure ORM using SQLAlchemy"""
+    """Configure ORM using SQLAlchemy."""
     from airflow.utils.log.secrets_masker import mask_secret
 
     log.debug("Setting up DB connection pool (PID %s)", os.getpid())
@@ -321,7 +266,7 @@ DEFAULT_ENGINE_ARGS = {
 
 
 def prepare_engine_args(disable_connection_pool=False, pool_class=None):
-    """Prepare SQLAlchemy engine args"""
+    """Prepare SQLAlchemy engine args."""
     default_args = {}
     for dialect, default in DEFAULT_ENGINE_ARGS.items():
         if SQL_ALCHEMY_CONN.startswith(dialect):
@@ -404,7 +349,7 @@ def prepare_engine_args(disable_connection_pool=False, pool_class=None):
 
 
 def dispose_orm():
-    """Properly close pooled database connections"""
+    """Properly close pooled database connections."""
     log.debug("Disposing DB connection pool (PID %s)", os.getpid())
     global engine
     global Session
@@ -418,13 +363,13 @@ def dispose_orm():
 
 
 def reconfigure_orm(disable_connection_pool=False, pool_class=None):
-    """Properly close database connections and re-configure ORM"""
+    """Properly close database connections and re-configure ORM."""
     dispose_orm()
     configure_orm(disable_connection_pool=disable_connection_pool, pool_class=pool_class)
 
 
 def configure_adapters():
-    """Register Adapters and DB Converters"""
+    """Register Adapters and DB Converters."""
     from pendulum import DateTime as Pendulum
 
     if SQL_ALCHEMY_CONN.startswith("sqlite"):
@@ -448,7 +393,7 @@ def configure_adapters():
 
 
 def validate_session():
-    """Validate ORM Session"""
+    """Validate ORM Session."""
     global engine
 
     worker_precheck = conf.getboolean("celery", "worker_precheck", fallback=False)
@@ -468,14 +413,11 @@ def validate_session():
 
 
 def configure_action_logging() -> None:
-    """
-    Any additional configuration (register callback) for airflow.utils.action_loggers
-    module
-    """
+    """Any additional configuration (register callback) for airflow.utils.action_loggers module."""
 
 
 def prepare_syspath():
-    """Ensures that certain subfolders of AIRFLOW_HOME are on the classpath"""
+    """Ensure certain subfolders of AIRFLOW_HOME are on the classpath."""
     if DAGS_FOLDER not in sys.path:
         sys.path.append(DAGS_FOLDER)
 
@@ -520,30 +462,38 @@ def get_session_lifetime_config():
 
 
 def import_local_settings():
-    """Import airflow_local_settings.py files to allow overriding any configs in settings.py file"""
+    """Import airflow_local_settings.py files to allow overriding any configs in settings.py file."""
     try:
         import airflow_local_settings
 
         if hasattr(airflow_local_settings, "__all__"):
-            for i in airflow_local_settings.__all__:
-                globals()[i] = getattr(airflow_local_settings, i)
+            names = list(airflow_local_settings.__all__)
         else:
-            for k, v in airflow_local_settings.__dict__.items():
-                if not k.startswith("__"):
-                    globals()[k] = v
+            names = list(filter(lambda n: not n.startswith("__"), airflow_local_settings.__dict__.keys()))
 
-        # TODO: Remove once deprecated
-        if "policy" in globals() and "task_policy" not in globals():
+        if "policy" in names and "task_policy" not in names:
             warnings.warn(
                 "Using `policy` in airflow_local_settings.py is deprecated. "
                 "Please rename your `policy` to `task_policy`.",
-                DeprecationWarning,
+                RemovedInAirflow3Warning,
                 stacklevel=2,
             )
-            globals()["task_policy"] = globals()["policy"]
-            del globals()["policy"]
+            setattr(airflow_local_settings, "task_policy", airflow_local_settings.policy)
+            names.remove("policy")
 
-        if not hasattr(task_instance_mutation_hook, "is_noop"):
+        plugin_functions = policies.make_plugin_from_local_settings(
+            POLICY_PLUGIN_MANAGER, airflow_local_settings, names
+        )
+
+        for name in names:
+            # If we have already handled a function by adding it to the plugin, then don't clobber the global
+            # function
+            if name in plugin_functions:
+                continue
+
+            globals()[name] = getattr(airflow_local_settings, name)
+
+        if POLICY_PLUGIN_MANAGER.hook.task_instance_mutation_hook.get_hookimpls():
             task_instance_mutation_hook.is_noop = False
 
         log.info("Loaded airflow_local_settings from %s .", airflow_local_settings.__file__)
@@ -562,9 +512,13 @@ def import_local_settings():
 
 
 def initialize():
-    """Initialize Airflow with all the settings from this file"""
+    """Initialize Airflow with all the settings from this file."""
     configure_vars()
     prepare_syspath()
+    configure_policy_plugin_manager()
+    # Load policy plugins _before_ importing airflow_local_settings, as Pluggy uses LIFO and we want anything
+    # in airflow_local_settings to take precendec
+    load_policy_plugins(POLICY_PLUGIN_MANAGER)
     import_local_settings()
     global LOGGING_CLASS_PATH
     LOGGING_CLASS_PATH = configure_logging()
@@ -627,6 +581,8 @@ IS_K8S_OR_K8SCELERY_EXECUTOR = conf.get("core", "EXECUTOR") in {
     executor_constants.CELERY_KUBERNETES_EXECUTOR,
     executor_constants.LOCAL_KUBERNETES_EXECUTOR,
 }
+IS_K8S_EXECUTOR_POD = bool(os.environ.get("AIRFLOW_IS_K8S_EXECUTOR_POD", ""))
+"""Will be True if running in kubernetes executor pod."""
 
 HIDE_SENSITIVE_VAR_CONN_FIELDS = conf.getboolean("core", "hide_sensitive_var_conn_fields")
 
@@ -653,3 +609,12 @@ DASHBOARD_UIALERTS: list[UIAlert] = []
 AIRFLOW_MOVED_TABLE_PREFIX = "_airflow_moved"
 
 DAEMON_UMASK: str = conf.get("core", "daemon_umask", fallback="0o077")
+
+
+# AIP-52: setup/teardown (experimental)
+# This feature is not complete yet, so we disable it by default.
+_ENABLE_AIP_52 = os.environ.get("AIRFLOW_ENABLE_AIP_52", "false").lower() in {"true", "t", "yes", "y", "1"}
+
+# AIP-44: internal_api (experimental)
+# This feature is not complete yet, so we disable it by default.
+_ENABLE_AIP_44 = os.environ.get("AIRFLOW_ENABLE_AIP_44", "false").lower() in {"true", "t", "yes", "y", "1"}
