@@ -17,23 +17,33 @@
 """Mask sensitive information from logs"""
 from __future__ import annotations
 
-import collections
+import collections.abc
 import logging
 import re
 import sys
-from typing import Any, Dict, Iterable, List, TextIO, Tuple, TypeVar, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    Generator,
+    Iterable,
+    Iterator,
+    List,
+    TextIO,
+    Tuple,
+    TypeVar,
+    Union,
+)
 
 from airflow import settings
 from airflow.compat.functools import cache, cached_property
+from airflow.typing_compat import TypeGuard
 
-try:
-    # kubernetes provider may not be installed
+if TYPE_CHECKING:
     from kubernetes.client import V1EnvVar
-except ImportError:
-    V1EnvVar = type("V1EnvVar", (), {})  # keep mypy happy about the V1EnvVar check
 
-
-Redactable = TypeVar("Redactable", str, Dict[Any, Any], Tuple[Any, ...], List[Any])
+Redactable = TypeVar("Redactable", str, "V1EnvVar", Dict[Any, Any], Tuple[Any, ...], List[Any])
 Redacted = Union[Redactable, str]
 
 log = logging.getLogger(__name__)
@@ -99,9 +109,9 @@ def mask_secret(secret: str | dict | Iterable, name: str | None = None) -> None:
     _secrets_masker().add_mask(secret, name)
 
 
-def redact(value: Redactable, name: str | None = None) -> Redacted:
+def redact(value: Redactable, name: str | None = None, max_depth: int | None = None) -> Redacted:
     """Redact any secrets found in ``value``."""
-    return _secrets_masker().redact(value, name)
+    return _secrets_masker().redact(value, name, max_depth)
 
 
 @cache
@@ -117,6 +127,19 @@ def _secrets_masker() -> SecretsMasker:
     )
 
 
+@cache
+def _get_v1_env_var_type() -> type:
+    try:
+        from kubernetes.client import V1EnvVar
+    except ImportError:
+        return type("V1EnvVar", (), {})
+    return V1EnvVar
+
+
+def _is_v1_env_var(v: Any) -> TypeGuard[V1EnvVar]:
+    return isinstance(v, _get_v1_env_var_type())
+
+
 class SecretsMasker(logging.Filter):
     """Redact secrets from logs"""
 
@@ -184,40 +207,44 @@ class SecretsMasker(logging.Filter):
 
         return True
 
-    def _redact_all(self, item: Redactable, depth: int) -> Redacted:
-        if depth > self.MAX_RECURSION_DEPTH or isinstance(item, str):
+    # Default on `max_depth` is to support versions of the OpenLineage plugin (not the provider) which called
+    # this function directly. New versions of that provider, and this class itself call it with a value
+    def _redact_all(self, item: Redactable, depth: int, max_depth: int = MAX_RECURSION_DEPTH) -> Redacted:
+        if depth > max_depth or isinstance(item, str):
             return "***"
         if isinstance(item, dict):
-            return {dict_key: self._redact_all(subval, depth + 1) for dict_key, subval in item.items()}
+            return {
+                dict_key: self._redact_all(subval, depth + 1, max_depth) for dict_key, subval in item.items()
+            }
         elif isinstance(item, (tuple, set)):
             # Turn set in to tuple!
-            return tuple(self._redact_all(subval, depth + 1) for subval in item)
+            return tuple(self._redact_all(subval, depth + 1, max_depth) for subval in item)
         elif isinstance(item, list):
-            return list(self._redact_all(subval, depth + 1) for subval in item)
+            return list(self._redact_all(subval, depth + 1, max_depth) for subval in item)
         else:
             return item
 
-    def _redact(self, item: Redactable, name: str | None, depth: int) -> Redacted:
+    def _redact(self, item: Redactable, name: str | None, depth: int, max_depth: int) -> Redacted:
         # Avoid spending too much effort on redacting on deeply nested
         # structures. This also avoid infinite recursion if a structure has
         # reference to self.
-        if depth > self.MAX_RECURSION_DEPTH:
+        if depth > max_depth:
             return item
         try:
             if name and should_hide_value_for_key(name):
-                return self._redact_all(item, depth)
+                return self._redact_all(item, depth, max_depth)
             if isinstance(item, dict):
                 to_return = {
-                    dict_key: self._redact(subval, name=dict_key, depth=(depth + 1))
+                    dict_key: self._redact(subval, name=dict_key, depth=(depth + 1), max_depth=max_depth)
                     for dict_key, subval in item.items()
                 }
                 return to_return
-            elif isinstance(item, V1EnvVar):
+            elif _is_v1_env_var(item):
                 tmp: dict = item.to_dict()
                 if should_hide_value_for_key(tmp.get("name", "")) and "value" in tmp:
                     tmp["value"] = "***"
                 else:
-                    return self._redact(item=tmp, name=name, depth=depth)
+                    return self._redact(item=tmp, name=name, depth=depth, max_depth=max_depth)
                 return tmp
             elif isinstance(item, str):
                 if self.replacer:
@@ -228,9 +255,13 @@ class SecretsMasker(logging.Filter):
                 return item
             elif isinstance(item, (tuple, set)):
                 # Turn set in to tuple!
-                return tuple(self._redact(subval, name=None, depth=(depth + 1)) for subval in item)
+                return tuple(
+                    self._redact(subval, name=None, depth=(depth + 1), max_depth=max_depth) for subval in item
+                )
             elif isinstance(item, list):
-                return [self._redact(subval, name=None, depth=(depth + 1)) for subval in item]
+                return [
+                    self._redact(subval, name=None, depth=(depth + 1), max_depth=max_depth) for subval in item
+                ]
             else:
                 return item
         # I think this should never happen, but it does not hurt to leave it just in case
@@ -246,30 +277,68 @@ class SecretsMasker(logging.Filter):
             )
             return item
 
-    def redact(self, item: Redactable, name: str | None = None) -> Redacted:
+    def redact(self, item: Redactable, name: str | None = None, max_depth: int | None = None) -> Redacted:
         """Redact an any secrets found in ``item``, if it is a string.
 
         If ``name`` is given, and it's a "sensitive" name (see
         :func:`should_hide_value_for_key`) then all string values in the item
         is redacted.
         """
-        return self._redact(item, name, depth=0)
+        return self._redact(item, name, depth=0, max_depth=max_depth or self.MAX_RECURSION_DEPTH)
+
+    @cached_property
+    def _mask_adapter(self) -> None | Callable:
+        """Pulls the secret mask adapter from config.
+
+        This lives in a function here to be cached and only hit the config once.
+        """
+        from airflow.configuration import conf
+
+        return conf.getimport("logging", "secret_mask_adapter", fallback=None)
+
+    @cached_property
+    def _test_mode(self) -> bool:
+        """Pulls the unit test mode flag from config.
+
+        This lives in a function here to be cached and only hit the config once.
+        """
+        from airflow.configuration import conf
+
+        return conf.getboolean("core", "unit_test_mode")
+
+    def _adaptations(self, secret: str) -> Generator[str, None, None]:
+        """Yields the secret along with any adaptations to the secret that should be masked."""
+        yield secret
+
+        if self._mask_adapter:
+            # This can return an iterable of secrets to mask OR a single secret as a string
+            secret_or_secrets = self._mask_adapter(secret)
+            if not isinstance(secret_or_secrets, str):
+                # if its not a string, it must be an iterable
+                yield from secret_or_secrets
+            else:
+                yield secret_or_secrets
 
     def add_mask(self, secret: str | dict | Iterable, name: str | None = None):
         """Add a new secret to be masked to this filter instance."""
-        from airflow.configuration import conf
-
-        test_mode: bool = conf.getboolean("core", "unit_test_mode")
         if isinstance(secret, dict):
             for k, v in secret.items():
                 self.add_mask(v, k)
         elif isinstance(secret, str):
-            if not secret or (test_mode and secret in SECRETS_TO_SKIP_MASKING_FOR_TESTS):
+            if not secret or (self._test_mode and secret in SECRETS_TO_SKIP_MASKING_FOR_TESTS):
                 return
-            pattern = re.escape(secret)
-            if pattern not in self.patterns and (not name or should_hide_value_for_key(name)):
-                self.patterns.add(pattern)
+
+            new_mask = False
+            for s in self._adaptations(secret):
+                if s:
+                    pattern = re.escape(s)
+                    if pattern not in self.patterns and (not name or should_hide_value_for_key(name)):
+                        self.patterns.add(pattern)
+                        new_mask = True
+
+            if new_mask:
                 self.replacer = re.compile("|".join(self.patterns))
+
         elif isinstance(secret, collections.abc.Iterable):
             for v in secret:
                 self.add_mask(v, name)
@@ -286,11 +355,61 @@ class RedactedIO(TextIO):
 
     def __init__(self):
         self.target = sys.stdout
-        self.fileno = sys.stdout.fileno
+
+    def __enter__(self) -> TextIO:
+        return self.target.__enter__()
+
+    def __exit__(self, t, v, b) -> None:
+        return self.target.__exit__(t, v, b)
+
+    def __iter__(self) -> Iterator[str]:
+        return iter(self.target)
+
+    def __next__(self) -> str:
+        return next(self.target)
+
+    def close(self) -> None:
+        return self.target.close()
+
+    def fileno(self) -> int:
+        return self.target.fileno()
+
+    def flush(self) -> None:
+        return self.target.flush()
+
+    def isatty(self) -> bool:
+        return self.target.isatty()
+
+    def read(self, n: int = -1) -> str:
+        return self.target.read(n)
+
+    def readable(self) -> bool:
+        return self.target.readable()
+
+    def readline(self, n: int = -1) -> str:
+        return self.target.readline(n)
+
+    def readlines(self, n: int = -1) -> list[str]:
+        return self.target.readlines(n)
+
+    def seek(self, offset: int, whence: int = 0) -> int:
+        return self.target.seek(offset, whence)
+
+    def seekable(self) -> bool:
+        return self.target.seekable()
+
+    def tell(self) -> int:
+        return self.target.tell()
+
+    def truncate(self, s: int | None = None) -> int:
+        return self.target.truncate(s)
+
+    def writable(self) -> bool:
+        return self.target.writable()
 
     def write(self, s: str) -> int:
         s = redact(s)
         return self.target.write(s)
 
-    def flush(self) -> None:
-        return self.target.flush()
+    def writelines(self, lines) -> None:
+        self.target.writelines(lines)
