@@ -19,28 +19,28 @@
 
 /* global document */
 
-import React, { useRef } from 'react';
-import { createRoot } from 'react-dom/client';
-import createCache from '@emotion/cache';
-import { useSearchParams } from 'react-router-dom';
-import { Flex, Box, useDimensions } from '@chakra-ui/react';
+import React, { useRef } from "react";
+import { createRoot } from "react-dom/client";
+import createCache from "@emotion/cache";
+import { useSearchParams } from "react-router-dom";
+import { Flex, Box, useDimensions } from "@chakra-ui/react";
 
-import App from 'src/App';
+import App from "src/App";
 
-import DatasetsList from './List';
-import DatasetDetails from './Details';
-import Graph from './Graph';
+import DatasetsList from "./List";
+import DatasetDetails from "./Details";
+import Graph from "./Graph";
 
 // create shadowRoot
-const root = document.querySelector('#root');
-const shadowRoot = root?.attachShadow({ mode: 'open' });
+const root = document.querySelector("#root");
+const shadowRoot = root?.attachShadow({ mode: "open" });
 const cache = createCache({
   container: shadowRoot,
-  key: 'c',
+  key: "c",
 });
-const mainElement = document.getElementById('react-container');
+const mainElement = document.getElementById("react-container");
 
-const DATASET_URI = 'uri';
+const DATASET_URI = "uri";
 
 const Datasets = () => {
   const [searchParams, setSearchParams] = useSearchParams();
@@ -58,16 +58,28 @@ const Datasets = () => {
     setSearchParams(searchParams);
   };
 
-  const datasetUri = decodeURIComponent(searchParams.get(DATASET_URI) || '');
+  const datasetUri = decodeURIComponent(searchParams.get(DATASET_URI) || "");
 
   return (
-    <Flex alignItems="flex-start" justifyContent="space-between" ref={contentRef}>
+    <Flex
+      alignItems="flex-start"
+      justifyContent="space-between"
+      ref={contentRef}
+    >
       <Box minWidth="450px" height="100%" overflowY="auto">
-        {datasetUri
-          ? <DatasetDetails uri={datasetUri} onBack={onBack} />
-          : <DatasetsList onSelect={onSelect} />}
+        {datasetUri ? (
+          <DatasetDetails uri={datasetUri} onBack={onBack} />
+        ) : (
+          <DatasetsList onSelect={onSelect} />
+        )}
       </Box>
-      <Box flex={1} ref={graphRef} height="calc(100vh - 68px)" borderColor="gray.200" borderWidth={1}>
+      <Box
+        flex={1}
+        ref={graphRef}
+        height="calc(100vh - 68px)"
+        borderColor="gray.200"
+        borderWidth={1}
+      >
         <Graph
           selectedUri={datasetUri}
           onSelect={onSelect}
@@ -85,6 +97,6 @@ if (mainElement) {
   reactRoot.render(
     <App cache={cache}>
       <Datasets />
-    </App>,
+    </App>
   );
 }
