@@ -53,7 +53,9 @@ ALL_PROVIDER_YAMLS = load_package_data()
 
 def get_available_packages():
     """Get list of all available packages to build."""
-    provider_package_names = [provider["package-name"] for provider in ALL_PROVIDER_YAMLS]
+    provider_package_names = [
+        provider["package-name"] for provider in ALL_PROVIDER_YAMLS if not provider.get("suspended")
+    ]
     return [
         "apache-airflow",
         "docker-stack",
